Package: epiNEM
Type: Package
Title: epiNEM
Version: 1.36.0
Authors@R: c(person("Martin", "Pirkl",
    email = "martinpirkl@yahoo.de",
    role = c("aut", "cre")),
    person("Madeline", "Diekmann",
    role = c("aut")))
Author: Madeline Diekmann & Martin Pirkl
Maintainer: Martin Pirkl <martinpirkl@yahoo.de>
Description: epiNEM is an extension of the original Nested Effects
    Models (NEM). EpiNEM is able to take into account double
    knockouts and infer more complex network signalling pathways. 
    It is tailored towards large scale double knock-out screens.
Depends: R (>= 4.1)
License: GPL-3
Encoding: UTF-8
LazyData: true
biocViews: Pathways, SystemsBiology, NetworkInference, Network
RoxygenNote: 7.3.1
Imports: BoutrosLab.plotting.general, BoolNet, e1071, gtools, stats,
        igraph, utils, lattice, latticeExtra, RColorBrewer, pcalg,
        minet, grDevices, graph, mnem, latex2exp
VignetteBuilder: knitr
Suggests: knitr, RUnit, BiocGenerics, STRINGdb, devtools, rmarkdown,
        GOSemSim, AnnotationHub, org.Sc.sgd.db, BiocStyle
BugReports: https://github.com/cbg-ethz/epiNEM/issues
URL: https://github.com/cbg-ethz/epiNEM/
git_url: https://git.bioconductor.org/packages/epiNEM
git_branch: RELEASE_3_23
git_last_commit: 73fc1ee
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-06-02
NeedsCompilation: no
Packaged: 2026-06-02 18:24:23 UTC; biocbuild
Built: R 4.6.0; ; 2026-06-03 13:36:04 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(plot,epiNEM)
S3method(plot,epiScreen)
S3method(plot,epiSim)
export(AddLogicGates)
export(CreateExtendedAdjacency)
export(CreateRandomGraph)
export(CreateTopology)
export(ExtendTopology)
export(GenerateData)
export(HeatmapOP)
export(Mll)
export(SimEpiNEM)
export(epiAnno)
export(epiNEM)
export(epiScreen)
export(perm.rank.test)
export(rank.enrichment)
import(RColorBrewer)
import(e1071)
import(grDevices)
import(lattice)
import(latticeExtra)
import(minet)
import(pcalg)
import(stats)
import(utils)
importFrom(BoolNet,getPathToAttractor)
importFrom(BoolNet,loadNetwork)
importFrom(BoutrosLab.plotting.general,legend.grob)
importFrom(graph,adj)
importFrom(gtools,combinations)
importFrom(igraph,graph.adjacency)
importFrom(igraph,layout.fruchterman.reingold)
importFrom(igraph,plot.igraph)
importFrom(latex2exp,TeX)
importFrom(mnem,mnem)
importFrom(mnem,transitive.reduction)

# epiNEM

EpiNEM is a direct extension of the NEM framework. epiNEM focuses on the epistasis between three S-genes, namely two parents and one child. Epistasis occurs, when the single effects of the two parents do not explain their combined effect on the child. EpiNEM models this epistasis with five Boolean logical gates for a triplet A->C, B->C.

- A or B, OR: A has an effect on C, B has an effect on C, A+B has an effect on C
- A and B, AND: A has no effect on C, B has no effect on C, A+B has an effect on C
- (not A and B) or (A and not B), XOR: A has an effect on C, B has an effect on C, A+B has no effect on C
- not A and B, A masks the effect of B: A has no effect on C, B has an effect on C, A+B has no effect on C
- A and not B, B masks the effect of A: A has an effect on C, B has no effect on C, A+B has no effect on C

Install:
--------

```{r}
if (!requireNamespace("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("epiNEM")
```

Most recent (devel) version:

```r
install.packages("devtools")

library(devtools)

install_github("cbg-ethz/epiNEM")

library(epiNEM)
```

Check out the vignette for working examples and the reproduction of the publication.

```r
vignette(package="epiNEM")
```

Toy example:

```r
data <- matrix(sample(c(0,1), 100*4, replace = T), 100, 4)

colnames(data) <- c("A", "A.B", "B", "C")

rownames(data) <- paste("E", 1:100, sep = "_")

res <- epiNEM(data, method = "exhaustive")

plot(res)
```
## References

Pirkl M, Diekmann M, van der Wees M, Beerenwinkel N, Fröhlich H, Markowetz F (2017). "Inferring modulators of genetic interactions with epistatic nested effects models." PLOS Computational Biology, 13, 1-18. doi: 10.1371/journal.pcbi.1005496.

## optional functions

#' @noRd
GreedyHillClimber <- function(n, experiments, data2,
                              data0, mutants, init = NULL) {
    cat("\n")
    cat(n)
    maxLlh <- -Inf
    startModel <- CreateRandomGraph(experiments)
    nonSelfEdges <- which(diag(nrow(startModel)) < 1)

    ##--- Iteratively update model and find highest scoring logics ---#
    extendedModels <- includeLogic(startModel, experiments, mutants)
    while (length(extendedModels)==0) {
        startModel <- CreateRandomGraph(experiments)
        extendedModels <- includeLogic(startModel, experiments, mutants)
    }
    extendedModels <- unlist(extendedModels, recursive=FALSE)
    result <- sapply(extendedModels, Mll, data2, data0)
    mLLscores <- unlist(result["mLL",])
    score <- max(mLLscores)
    if (is.null(init[1])) {
        model <- startModel
    } else {
        model <- init
    }

    while (score > maxLlh) {
        cat('.')
        maxLlh <- score
        prevModel <- model
        nextGen <- FindNeighbours(model, nonSelfEdges)
        extendedModels <- lapply(nextGen, includeLogic, experiments, mutants)
        extendedModels2 <- unlist(unlist(extendedModels, recursive=FALSE),
                                  recursive=FALSE)
        result <- sapply(extendedModels2, Mll, data2, data0)
        mll <- unlist(result["mLL",])
        index <- which.max(mll)

        score <- mll[index]
        model <- extendedModels2[[index]]$origModel
    }
    return(list(maxLlh=maxLlh, model=prevModel))
}

#' @noRd
FindNeighbours <- function(model, edges) {
    ## Return all models that differ only one edge from the current model by
    ## removing or adding a non-self edge.
    ChangeEdge <- function(edge, model) {
        if (model[edge] == 1) {
            model[edge] = 0
        } else if (model[edge] == 0) {
            model[edge] = 1
        }
        return(model)
    }
    models <- lapply(edges, ChangeEdge, model)
    models <- unique(models)
    return(models)
}

#' @noRd
EnumerateModels <- function(size, nodes=NULL) {
    ## Enumerates all possible adjacency matrices for a given size.
    ## Returns a list of
    ## all unique transitively closed adjacency matrices.
    ## Function adapted from NEM package
    if (!size %in% 2:5) {
        stop("Enumeration only feasible for networks up to 5 nodes.\n")
    }
    if (length(nodes) == 0) {
        nodes <- LETTERS[1:size]
    }

    createModels <- function(bincom, size, nodes) {
        model <- diag(size)
        model[which(model == 0)] <- bincom
        dimnames(model) <- list(nodes, nodes)
        return(list(model))
    }

    asMatrix <- function(model, size, nodes) {
        model <- matrix(model, size)
        dimnames(model) <- list(nodes, nodes)
        return(list(model))
    }

    bincom <- bincombinations(size * (size - 1))
    models <- apply(bincom, 1, createModels, size, nodes)
    models <- unique(matrix(unlist(models), ncol=size * size, byrow=TRUE))
    models <- unlist(apply(models, 1, asMatrix, size, nodes), recursive=FALSE)

    ## cat("\nGenerated", length(models),
    ## "unique models out of", nrow(bincom), "\n")
    return(models)
}

## functions needed in order to run LogicNEM

#' Create a random graph
#' @description Returns a model graph with randomly sampled edges.
#' Every possible edge has a probability to exist in the graph.
#' @param pathwayGenes vector of genes in the pathway
#' @param edgeProb probability of random edge
#' @export
#' @examples
#' graph <- CreateRandomGraph(c("Ikk1", "Ikk2", "RelA"))
#' @return adjacency matrix
CreateRandomGraph <- function(pathwayGenes, edgeProb=0.5) {
    size     <- length(pathwayGenes)
    model    <- diag(size)
    nEdges   <- size * (size - 1)
    model[which(model == 0)] <-
        sample(0:1, nEdges, TRUE, c(1 - edgeProb, edgeProb))
    dimnames(model) <- list(pathwayGenes, pathwayGenes)
    return(model)
}

get_col <- function(i, m) ((i-1) %/% nrow(m)) + 1
get_row <- function(i, m) (i-1) %% nrow(m) + 1

#' Evaluation of graphs
#' @param Phi model to be evaluated
#' @param D1 observed data matrix
#' @param D0 complementary D1
#' @param ltype likelihood type either "marginal" or "maximum"
#' @param para false positive and false negative rates
#' @description Computes marginal log-likelihood
#' for model Phi given observed data matrix D1
#' @export
#' @examples
#' Phi <- matrix(sample(c(0,1), 9, replace = TRUE), 3, 3)
#' data <- matrix(sample(c(0,1), 3*10, replace = TRUE), 10, 3)
#' rownames(Phi) <- colnames(Phi) <- colnames(data) <- c("Ikk1", "Ikk2", "RelA")
#' score <- Mll(Phi, D1 <- data, D0 <- 1 - data)
#' @return list with likelihood poster probability, egene positions
Mll <- function(Phi, D1, D0, ltype = "marginal", para = c(0.13, 0.05)) {
    ## Computes marginal log-likelihood for model Phi,
    ## observed data matrix D1, and
    ## complementary data matrix D0
    ## Function adapted from NEM package
    if (is.matrix(Phi)) {
        Phi2 <- Phi
    } else {
        Phi2 <- as.matrix(Phi$model)
    }
    Phi2 <- Phi2[colnames(D1), ]
    L <- para[1]^(D1 %*% (1 - Phi2)) *
                     (1 - para[1])^(D0 %*% (1 - Phi2)) *
                                       (1 - para[2])^(D1 %*% Phi2) *
                                                         para[2]^(D0 %*% Phi2)
    posterior <- L / (rowSums(L))
    LLperGene <- log(rowSums(L))
    mLL       <- sum(LLperGene)
    theta     <- apply(posterior, 1, function(e) e == max(e))
    mappos    <- apply(theta, 1, which)
    ## map instead of mll
    if (ltype %in% "maximum") {
        mLL <- sum(apply(L, 1, max))
    }
    return(list(posterior=posterior,
                LLperGene=LLperGene,
                mLL=mLL,
                mappos=mappos,
                para=para))
}

#' @noRd
getStarters <- function(mutants, experiments){
    ## get positions of doublemutants
    mutantslist <- strsplit(mutants, ".", fixed=TRUE)
    doublepos <- c()
    for (i in 1:length(mutantslist)){
        if (length(unlist(mutantslist[i])) == 2)
            doublepos <- c(doublepos, i)
    }
    starters <- c()
    dbvec <- c()
    ## create startStates for each knockout
    for (i in 1:length(experiments)){
        if (i %in% doublepos){
            for (j in experiments){
                if (j %in% unlist(mutantslist[doublepos]))
                    dbvec <- c(dbvec, 1)
                else dbvec <- c(dbvec, 0)
            }
            starters <- c(starters, dbvec)
        }
        z1 <- i-1
        z1 <- rep(0,z1)
        z2 <- length(experiments)-i
        z2 <- rep(0, z2)
        vector <- c(z1, 1, z2)
        starters <- c(starters, vector)
    }
    if (length(dbvec)==0){
        for (j in experiments){
            if (j %in% unlist(mutantslist[length(mutantslist)]))
                dbvec <- c(dbvec, 1)
            else dbvec <- c(dbvec, 0)
        }
        starters <- c(starters, dbvec)
    }
    return(starters)
}

#' Create an extended adjacency matrix
#' @description extend adjacency matrices taking cycles and logics into account.
#' For every given start state, the final state
#' is computed yu using BoolNet.
#' @param network network created by BoolNet from file
#' @param mutants vector of single knockouts
#' @param experiments vector of all knockouts
#' @importFrom BoolNet getPathToAttractor
#' @import stats
#' @export
#' @examples
#' library(BoolNet)
#' data(cellcycle)
#' extModel <- CreateExtendedAdjacency(cellcycle,
#' c(cellcycle$genes, "CycD.Rb"), cellcycle$genes)
#' @return extended adjacency matrix
CreateExtendedAdjacency <- function(network, mutants, experiments){
    starters <- matrix(getStarters(mutants, experiments),
                       length(mutants), byrow=TRUE)
    rownames(starters) <- mutants
    colnames(starters) <- experiments

    ## get steady state of each startState
    getMyAttractor <- function(e, network, includeAttractorStates) {
        network$fixed[which(e == 1)] <- 1
        return(getPathToAttractor(network, e, includeAttractorStates))
    }
    data <- apply(starters, 1, getMyAttractor, network,
                  includeAttractorStates="all")

    ## get observed data by taking final state from all the random startStates.
    extadj <- data[[1]][nrow(data[[1]]),]
    for (i in 2:length(data)){
        d <- data[[i]][nrow(data[[i]]),]
        extadj <- rbind(extadj, d);
    }
    rownames(extadj) <- mutants
    return(extadj)
}

#' @noRd
includeLogic <- function(adj, experiments, mutants){
    if (is.list(adj)) {
        adj=matrix(unlist(adj),length(experiments), byrow=FALSE)
    }
    rownames(adj) <- experiments
    colnames(adj) <- rownames(adj)
    diag(adj)=0
    adj <- adj[order(rownames(adj)), order(colnames(adj))]
    mutantslist <- strsplit(mutants, ".", fixed=TRUE)
    doublepos <- c()
    for (i in 1:length(mutantslist)) {
        if (length(unlist(mutantslist[i])) == 2) {
            doublepos <- c(doublepos, i)
        }
    }
    ## look for suitable triples and create logic vector depending
    ## on the relationship of the parents
    notriples <- 0
    logic <- c()
    liste <- list()
    column <- c()
    for (c in 1:length(experiments)) {
        singles <- c()
        parents <- names(which(adj[,c]==1))
        if ((length(parents) == 2) &&
            ##check whether double mutant is available in the data
            (parents[1] %in% unlist(mutantslist[doublepos])) &&
            parents[2] %in% unlist(mutantslist[doublepos])) {
            for (i in 1:length(parents)) {
                singles <- cbind(singles, parents[i])
            }
            notriples <- notriples+1
            relation <- adj[singles, singles]
            diag(relation) <- 0
            NOT2 <- paste(parents[2], "masks the effect of", parents[1])
            NOT1 <- paste(parents[1], "masks the effect of", parents[2])
            if (sum(relation) == 0) {
                logic <- c("OR", "XOR", "AND", NOT1, NOT2)
            }
            if (relation[2] == 1) {
                logic <- c(NOT2)
            }
            if (relation[3] == 1) {
                logic <- c(NOT1)
            }
            liste[[notriples]] <- logic
            column <- cbind(column, c)
        }
        if (length(parents) == 2 &
            !(all(parents %in% unlist(mutantslist[doublepos])))) {
            notriples <- notriples + 1
            liste[[notriples]] <- "OR"
            column <- cbind(column, c)
        }
        if (length(parents) > 2) {
            notriples <- notriples + 1
            liste[[notriples]] <- "OR"
            column <- cbind(column, c)
        }
        if (length(parents) == 1) {
            notriples <- notriples + 1
            liste[[notriples]] <- "OR"
            column <- cbind(column, c)
        }
    }
    if (length(liste) > 0) {
        logicmatrix <- as.matrix(expand.grid(liste))
        ## create logics file from adjacency matrix
        ## using logics provided by logic vector
        ## ready for using BoolNet
        randomnames <- sort(runif(nrow(logicmatrix)))
        networks <- list()
        for (modelno in 1:nrow(logicmatrix)) {
            lo <- 0
            network <- character()
            countline <- 1
            network[countline] <- "targets, factors"
            for (c in experiments) {
                tmp <- NULL
                count <- 1
                if (sum(adj[, which(colnames(adj) %in% c)]) == 0) {
                    tmp <- paste(tmp, paste(c, ", ", c, sep=""), sep = "")
                } else {
                    tmp <- paste(tmp, paste(c, ", ", sep=""), sep = "")
                }
                count2 <- 0
                count3 <- 0
                for (r in experiments) {
                    if (adj[r,c]==1) {
                        if ((count==1) &&
                            (which(rownames(adj)==c) %in% column)) {
                            help <- r
                            count <- count+1
                            if (sum(adj[, c]) == 1) {
                                tmp <- paste(tmp, paste(r, sep=""), sep = "")
                                lo <- lo + 1
                            }
                        }
                        else if ((count == 2) &&
                                 (which(rownames(adj)==c) %in% column)) {
                            NOT2 <- paste(r, "masks the effect of", help)
                            NOT1 <- paste(help, "masks the effect of", r)
                            if (count3 < 1) {
                                lo <- lo+1
                            }
                            if (logicmatrix[modelno, lo]=="OR" & count3 == 0) {
                                tmp <-
                                    paste(tmp, paste(help, " | ", r, sep=""),
                                          sep = "")
                                count3 <- count3 + 1
                            } else if
                            (logicmatrix[modelno, lo]=="OR" & count3 > 0) {
                                tmp <-
                                    paste(tmp,
                                          paste(" | ", help, " | ", r, sep=""),
                                          sep = "")
                            } else if
                            (logicmatrix[modelno, lo]=="AND") {
                                tmp <-
                                    paste(tmp,
                                          paste("(", help, " & ", r, ")",
                                                sep=""),
                                          sep = "")
                            } else if
                            (logicmatrix[modelno, lo]=="XOR") {
                                tmp <-
                                    paste(tmp,
                                          paste("( ", help, " & ! ", r
                                               ,") | (", r, " & ! ", help, ")",
                                                sep = ""),
                                          sep = "")
                                ## help refers to the first element
                            } else if
                            (logicmatrix[modelno, lo]==NOT2) {
                                tmp <-
                                    paste(tmp,
                                          paste("(", help, " & ! ", r, ")",
                                                sep=""),
                                          sep = "")
                            } else if
                            (logicmatrix[modelno, lo]==NOT1) {
                                tmp <-
                                    paste(tmp,
                                          paste("(", r, " & ! ", help, ")",
                                                sep=""),
                                          sep = "")
                            } else {
                                count2 <- count2 + 1
                                if
                                (count2 <
                                 sum(adj[, which(colnames(adj) %in% c)])) {
                                    tmp <- paste(tmp,
                                                 paste(r, " | ",
                                                       sep=""),
                                                 sep = "")
                                } else {
                                    tmp <- paste(tmp, paste(r,
                                                            sep=""),
                                                 sep = "")
                                    lo <- lo + 1
                                }
                            }
                        }
                    }
                }
                countline <- countline + 1
                network[countline] <- tmp
            }
            networks[[modelno]] <- network # write(network, file = path)
        }
        test <- lapply(1:nrow(logicmatrix),
                       function(x) getExtendedAdjacency(x,
                                                        logicmatrix,
                                                        column,
                                                        adj,
                                                        mutants,
                                                        experiments,
                                                        networks))
        return(test)
    }
}

#' @noRd
net2bool <- function(network) {
    parseBooleanFunction <- get("parseBooleanFunction",
                                envir = asNamespace("BoolNet"))
    booln <- list()
    logics <- network[-1]
    booln$genes <- gsub(",.*", "", logics)
    booln$fixed <- rep(-1, length(booln$genes))
    names(booln$fixed) <- booln$genes
    booln$interactions <- list()
    for (i in 1:length(booln$genes)) {
        tmp <- unlist(strsplit(gsub("\\(|\\)|\\&|!|\\|",
                                    "", gsub(".*,", "", logics[i])), " "))
        if (sum(tmp %in% "") > 0) { tmp <- tmp[-which(tmp %in% "")] }
        tmp <- unique(tmp)
        booln$interactions[[booln$genes[[i]]]] <- list()
        booln$interactions[[booln$genes[[i]]]][["input"]] <-
            which(booln$genes %in% tmp)
        booln$interactions[[booln$genes[[i]]]][["expression"]] <-
            gsub(".*, ", "", logics[i])
        booln$interactions[[booln$genes[[i]]]][["func"]] <-
            .Call("getTruthTable_R",
                  parseBooleanFunction(
                      booln$interactions
                      [[booln$genes[[i]]]][["expression"]],
                      booln$genes), as.integer(3), PACKAGE="BoolNet")[[2]]
    }
    class(booln) <- "BooleanNetwork"
    return(booln)
}

#' create with logics extended adjacency matrix
#' @importFrom BoolNet loadNetwork
#' @noRd
getExtendedAdjacency <-function(modelno, logicmatrix,
                                column, adj, mutants,
                                experiments, networks) {
    network <- net2bool(networks[[modelno]])
    extadj2 <- CreateExtendedAdjacency(network, unique(mutants), experiments)
    return(list(list(origModel=adj, model=extadj2,
                     logics=logicmatrix[modelno,], column=column)))
}

#' @noRd
AttachEGenes <- function(posterior, experiments){
    maxpost <-
        lapply(1:nrow(posterior),
               function(x) length(which(posterior[x,]==max(posterior[x,]))))
    attachedEs <- matrix()
    names <- c()
    attachedEsADD <- 0
    namesADD <- c()
    Egeneset <- matrix()
    Egeneset <- NULL
    EgeneADD <- 0
    EgeneADD <- NULL
    j <- 1
    k <- 1
    Epos <- c()
    for (i in 1:length(maxpost)){
        if (maxpost[i]==1){
            Epos <- cbind(Epos, i)
            attachedEs[j] <- names(which.max(posterior[i,]))
            names[j] <- rownames(posterior)[i]
            j <- j+1
        } else {
            ## for (e in 1:unlist(maxpost[i])){
            attachedEsADD <- attachedEsADD+1
            ##   namesADD[k] <- rownames(posterior)[i]
            ##   k <- k+1
            ## }
        }
    }
    attachedEs=as.matrix(attachedEs)
    rownames(attachedEs) <- names
    attachedEsADD=as.matrix(attachedEsADD)
    rownames(attachedEsADD) <- namesADD

    for (i in experiments){
        if (is.na(table(attachedEs)[i])){
            tablt=0
        } else tablt=table(attachedEs)[i]
        ## if (is.na(table(attachedEsADD)[i])){
        Egeneset <- rbind(Egeneset, tablt)
        ## } else Egeneset <- rbind(Egeneset,
        ## paste(tablt, "+", table(attachedEsADD)[i], sep=""))
    }
    Egeneset <- rbind(Egeneset, attachedEsADD)

    rownames(Egeneset)= c(experiments, "null")
    colnames(Egeneset)="noE"
    return(Egeneset)
}

#' Create Topology.
#'
#' Create topology for a randomly generated pathway topology
#' @param single number of single knockouts
#' @param double number of double knockouts
#' @param force if true the random model will have a sophisticated logical gate
#' @export
#' @examples
#' model <- CreateTopology(3, 1)
#' @return adjacency matrix
CreateTopology <- function(single, double, force = TRUE) {
    extendedModels <- list()
    singleKOs <- LETTERS[1:single]
    experiments <- singleKOs
    doubleKOs <- lapply(1:double, GenerateDoubleKO, singleKOs)
    doubleKOs <- unlist(unique(doubleKOs))
    mutants   <- sort(c(singleKOs, doubleKOs))
    donotextend <- FALSE

    while (length(extendedModels)==0){
        startModel   <- CreateRandomGraph(singleKOs)
        startModel <- startModel[order(apply(startModel, 1, sum),
                                       decreasing = TRUE),
                                 order(apply(startModel, 1, sum),
                                       decreasing = TRUE)]
        startModel[lower.tri(startModel)] <- 0
        startModel <- startModel[order(rownames(startModel)),
                                 order(colnames(startModel))]
        diag(startModel) <- 0
        if (force) {
            if (sum(apply(startModel, 2, sum) >= 2) > 0) {
                if (sum(startModel[,which(
                    apply(startModel, 2, sum) >= 2)[1]] == 1) > 0) {
                    if (!(paste(
                             rownames(
                                 startModel)[which(
                                         startModel[, which(
                                             apply(startModel, 2, sum) >= 2)[1]]
                                         == 1)[1:2]], collapse = ".") %in%
                          mutants)) {
                        mutants <-
                            sort(c(singleKOs,
                                   paste(rownames(
                                       startModel)[which(startModel[,which(
                                                     apply(startModel,
                                                           2,
                                                           sum) >= 2)[1]]
                                                     == 1)[1:2]],
                                       collapse = ".")))
                    }
                    donotextend <- FALSE
                } else {
                    donotextend <- TRUE
                }
            } else {
                donotextend <- TRUE
            }
        }
        if (!donotextend) {
            extendedModels <- includeLogic(startModel, experiments, mutants)
            ## extendedModels <- unlist(extendedModels, recursive=FALSE)
        }
    }
    if (length(extendedModels) == 5) {
        prob <- c(0.018, 0.49, 0.49, 0.001, 0.001)
    } else {
        prob <- rep(1/length(extendedModels), length(extendedModels))
    }

    selectedModel <- sample(1:length(extendedModels), 1, prob = prob)
    topology      <- extendedModels[[selectedModel]]

    return(topology)
}

#' Generate double knock-out.
#'
#' Generate a random double mutant for artificial data
#' @param singleKOs: vector of single mutants
#' @importFrom gtools combinations
#' @noRd
GenerateDoubleKO <- function(d, singleKOs) {
    allDoubles    <- combinations(length(singleKOs), 2, singleKOs)
    doubleKO      <- allDoubles[d, ]
    doubleKO      <- do.call(paste, as.list(c(doubleKO, sep=".")))
    return(doubleKO)
}

#' AUC permutation test
#'
#' computes the area under the rank enrichment score curve and does
#' a permutation test to compute the p-value
#' @param x numeric vector of ranks
#' @param y numeric vector of the superset of x
#' @param alternative character for test type: 'less','greater','two.sided'
#' @param iter integer number of iterations
#' @return p-value
#' @author Martin Pirkl
#' @export
#' @examples
#' x <- 1:10
#' y <- 1:100
#' perm.rank.test(x,y,alternative='less')
#' perm.rank.test(x,y,alternative='greater')
perm.rank.test <- function(x,y=NULL,
                 alternative=c("two.sided", "less", "greater"),iter=1000) {
    xscore0 <- c(1,sort(x),length(y))
    scorel <- sort(x)/length(y)-
        (1:length(x))/length(x)
    score0 <- c(0,scorel,0)
    aucr <-
        sum(((score0[-length(score0)]+
                  score0[-1])/2)*(xscore0[-1]-
                                      xscore0[-length(xscore0)]))
    scorer <- (1:length(x))/length(x)-
        sort(x)/length(y)
    score0 <- c(0,scorer,0)
    aucl <- sum(((score0[-length(score0)]+
                      score0[-1])/2)*(xscore0[-1]-
                                          xscore0[-length(xscore0)]))
    count <- c(0,0)
    for (i in seq_len(iter)) {
        x.p <- sample(y,length(x))
        xscore0 <- c(1,sort(x.p),length(y))
        scorel <- sort(x.p)/length(y)-
            (1:length(x.p))/length(x.p)
        score0 <- c(0,scorel,0)
        aucr.p <-
            sum(((score0[-length(score0)]+
                      score0[-1])/2)*(xscore0[-1]-
                                          xscore0[-length(xscore0)]))
        scorer <- (1:length(x.p))/length(x.p)-
            sort(x.p)/length(y)
        score0 <- c(0,scorer,0)
        aucl.p <- sum(((score0[-length(score0)]+
                            score0[-1])/2)*(xscore0[-1]-
                                                xscore0[-length(xscore0)]))
        if (aucl<=aucl.p) {
            count[1] <- count[1]+1
        }
        if (aucr<=aucr.p) {
            count[2] <- count[2]+1
        }
    }
    p.values <- count/iter
    if (alternative[1]=='less') {
        return(list(p.value=p.values[1]))
    } else if (alternative[1]=='greater') {
        return(list(p.value=p.values[2]))
    } else if (alternative[1]=='two.sided') {
        p.tmp <- 2*min(p.values)
        return(list(p.value=p.tmp))
    }
}
#' Rank enrichment
#'
#' Infers a signalling pathway from peerturbation experiments.
#' @param data m times l matrix with m observed genes and l
#' variables with numeric values to rank the genes
#' @param list list of of vectors of genes
#' @param list2 optional list with same length as list
#' @param n length of the gradient (maximum: m)
#' @param main character string for main header; if NULL uses
#' the column names of data by default
#' @param col1 color of the gradient
#' @param col2 color of the first list
#' @param col3 color of the second list2
#' @param blim numeric vector of length two with the lower
#' and upper bounds for the gradient
#' @param p numeric adjustment (length four) of the left side of the
#' gradient (low means more to the left, high more to the right)
#' the right side of the enrichment lines and the top positions
#' of the additional matrices in case of vis='matrices'
#' @param lwd line width of the enrichment lines
#' @param test test function for the enrichment p-value; must
#' have input argument and output values same as perm.rank.test;
#' e.g., wilcox.test or ks.test (here 'less' and 'greater' are switched!)
#' @param vis method for visualisation: 'matrix' uses one
#' matrix heatmap for; 'matrices' uses several matrices
#' (experimental), 'colside'
#' uses the colSideColors argument for the ticks of genes in
#' list/list2 (can use a lot of memory; experimental)
#' @param verbose if TRUE gives prints additional output
#' @param ... additional arguments for epiNEM::HeatmapOP
#' @return transitively closed matrix or graphNEL
#' @author Martin Pirkl
#' @export
#' @import lattice
#' @importFrom latex2exp TeX
#' @examples
#' data <- matrix(rnorm(100*2),100,2)
#' rownames(data) <- 1:100
#' colnames(data) <- LETTERS[1:2]
#' list <- list(first = as.character(sample(1:100, 10)), second = as.character(sample(1:100, 20)))
#' rank.enrichment(data,list)
rank.enrichment <- function(data,list,list2=NULL,n=1000,main=NULL,col1="RdBu",
                            col2=rgb(1,0,0,0.75),col3=rgb(0,0,1,0.75),
                            blim=NULL,p=NULL,lwd=3,
                            test=wilcox.test,vis="matrix",
                            verbose=FALSE,...) {
    if (is.null(p[1])) {
        p <- c(0,1,0.466,0.333)
    }
    pvals <- matrix(NA,ncol(data),length(list))
    ylim <- c(-0,1)
    if (is.null(list2[1])) {
        pvals <- matrix(NA,length(list)*ncol(data),3)
        colnames(pvals) <- c("less","greater","two.sided")
    } else {
        pvals <- matrix(NA,length(list)*ncol(data),2*3)
        colnames(pvals) <- rep(c("less","greater","two.sided"),2)
    }
    rownames(pvals) <- rep(colnames(data),each=length(list))
    for (i in 1:ncol(data)) {
        data.tmp <- sort(data[,i],decreasing=FALSE)
        if (is.null(blim[1])) {
            blim2 <- c(-max(abs(data.tmp)),max(abs(data.tmp)))
        } else {
            blim2 <- blim
        }
        for (j in 1:length(list)) {
            tmp <- match(list[[j]],names(data.tmp))
            tmp <- tmp[which(is.na(tmp) == FALSE)]
            pvals[j+length(list)*(i-1),1] <-
                test(tmp,rank(data.tmp),alternative="less")$p.value
            pvals[j+length(list)*(i-1),2] <-
                test(tmp,rank(data.tmp),alternative="greater")$p.value
            pvals[j+length(list)*(i-1),3] <-
                2*min(pvals[j+length(list)*(i-1),1:2])
            mat.tmp <- rbind(data.tmp,NA)
            mat.tmp[2,tmp] <- blim2[1]
            if (vis == 'colside') {
                csc1 <- mat.tmp[2,]
                csc1[tmp] <- col2
                csc1[is.na(csc1)] <- "transparent"
            }
            xscore0 <- c(1,sort(tmp),length(data.tmp))
            ## left:
            scorel <- sort(tmp)/length(data.tmp)-
                (1:length(tmp))/length(tmp)
            score0 <- c(0,scorel,0)
            aucl <-
                sum(((score0[-length(score0)]+
                          score0[-1])/2)*(xscore0[-1]-
                                              xscore0[-length(xscore0)]))
            ## right:
            scorer <- (1:length(tmp))/length(tmp)-
                sort(tmp)/length(data.tmp)
            score0 <- c(0,scorer,0)
            aucr <- sum(((score0[-length(score0)]+
                              score0[-1])/2)*(xscore0[-1]-
                                                  xscore0[-length(xscore0)]))
            if (aucl >= aucr) {
                score <- scorel
            } else {
                score <- scorer
            }
            ## two-sided?:
            score <- abs(score)
            ## all from here:
            score0 <- c(0,score,0)
            xscore0 <- c(1,sort(tmp),length(data.tmp))
            ## auc:
            auc <- sum(((score0[-length(score0)]+
                             score0[-1])/2)*(xscore0[-1]-
                                                 xscore0[-length(xscore0)]))
            if (!is.null(list2[1])) {
                tmp2 <- tmp
                tmp <- match(list2[[j]],names(data.tmp))
                tmp <- tmp[which(is.na(tmp) == FALSE)]
                pvals[j+length(list)*(i-1),4] <-
                    test(tmp,rank(data.tmp),alternative="less")$p.value
                pvals[j+length(list)*(i-1),5] <-
                    test(tmp,rank(data.tmp),alternative="greater")$p.value
                pvals[j+length(list)*(i-1),6] <-
                    test(tmp,rank(data.tmp))$p.value
                mat.tmp <- rbind(mat.tmp,NA)
                mat.tmp[3,tmp] <- blim2[2]
                if (vis == 'colside') {
                    csc2 <- mat.tmp[3,]
                    csc2[tmp] <- col3
                    csc2[is.na(csc2)] <- "transparent"
                }
                score <- sort(tmp)/length(data.tmp)-
                    (1:length(tmp))/length(tmp)
                score <- c(0,abs(score),0)
                xscore <- c(1,sort(tmp),length(data.tmp))
                tmp <- c(tmp,tmp2)
            } else {
                score <- score0
                xscore <- xscore0
                col3 <- "transparent"
            }
            select <- sort(c(tmp,(1:n)*length(data.tmp)/n))
            mat.tmp <- mat.tmp[,select]
            csc0 <- NULL
            if (vis == 'colside') {
                csc1 <- csc1[select]
                csc2 <- csc2[select]
                csc0 <- rep("transparent",length(csc1))
                csc0[which(csc1 != "transparent")] <- col2
                csc0[which(csc2 != "transparent")] <- col3
                csc0[which(csc1 != "transparent" & csc2 != "transparent")] <-
                    "black"
            }
            colnames(mat.tmp) <- NULL
            rownames(mat.tmp) <- NULL
            if (is.null(main[1])) {
                main2 <- colnames(data)[i]
            } else {
                main2 <- main
            }
            if (is.null(list2[1])) {
                p.text <- c(paste0(gsub("_"," ",names(list)[j]), " (p-value: ",
                                   format(pvals[j+length(list)*(i-1),2],digits=3),
                                   " (greater), ",
                                   format(pvals[j+length(list)*(i-1),1],
                                          digits=3),
                                   " (less))"))
            } else {
                p.text <- c(paste0(gsub("_"," ",names(list)[j]), " (p-value: ",
                                   format(pvals[j+length(list)*(i-1),2],
                                          digits=3),
                                   " (greater), ",
                                   format(pvals[j+length(list)*(i-1),1],
                                          digits=3),
                                   " (less))"),
                            paste0(gsub("_"," ",names(list2)[j]),
                                   " (p-value: ",
                                   format(pvals[j+length(list)*(i-1),5],
                                          digits=3), " (greater), ",
                                   format(pvals[j+length(list)*(i-1),4],
                                          digits=3)," (less))"))
            }
    a <- lattice::xyplot(score0~xscore0,
                         strip=FALSE,type="l",ylim=ylim,
                         ylab="enrichment score",xlab="",col=col2,
                         key=list(corner=c(0.5,1),
                                  lines=list(col=c(col2,col3),
                                             lty=c(1,1),lwd=10),
    text=list(p.text)),
    lwd=lwd,par.settings=list(axis.line=list(lwd=0)),
    scales=list(x=list(draw=FALSE)),
    panel = function(...) {
        panel.xyplot(...)
        panel.abline(h=seq(-0.8,0.8,length.out=9),lty=3,col=rgb(0,0,0,0.75))
        panel.xyplot(x=xscore,y=score,type="l",lwd=lwd,col=col3)
    },
    main =  main2
            )
            if (!is.null(list2[1])) {
                sub <- ""
            } else {
                sub <- NULL
            }
    more <- FALSE
            if (vis == 'colside') {
                mat.tmp <- mat.tmp[1,]
            } else if (vis == 'matrices') {
                mat.tmp1 <- mat.tmp[2:3,]
                mat.tmp1[which(is.na(mat.tmp1)==TRUE)] <- blim2[2]
                mat.tmp1[2,] <- NA
                if (!is.null(list2[1])) {
                    mat.tmp2 <- mat.tmp[3,,drop=FALSE]
                    mat.tmp2[which(is.na(mat.tmp2)==TRUE)] <- blim2[1]
                }
                more <- TRUE
                mat.tmp[2:3,] <- NA
            }
            b <- HeatmapOP(mat.tmp,Colv=0,Rowv=0,
                                   bordercol="transparent",col=col1,
                                   borderwidth=0,colNA="transparent",
                                   breaks=seq(blim2[1],blim2[2],
                                              length.out=100),
                                   sub = sub, colSideColors = csc0, ...)
            print(a,position=c(0,0.5,p[2],1),more=TRUE)
            print(b,position=c(p[1],0,1,0.6),more=more)
            if (vis == 'matrices') {
                c1 <- HeatmapOP(mat.tmp1,Colv=0,Rowv=0,
                                bordercol="transparent",
                                col=c(col2,"white"),
                                borderwidth=0,colNA="transparent",
                                breaks=seq(blim2[1],blim2[2],
                                           length.out=100),
                                sub = sub, ...)
                more2 <- FALSE
                if (!is.null(list2[1])) {
                    c2 <- HeatmapOP(mat.tmp2,Colv=0,Rowv=0,
                                    bordercol="transparent",
                                    col=c("white",col3),
                                    borderwidth=0,colNA="transparent",
                                    breaks=seq(blim2[1],blim2[2],
                                               length.out=100),
                                    sub = sub, ...)
                    more2 <- TRUE
                }
                print(c1,position=c(p[1],0,1,p[3]),more=more2)
                if (!is.null(list2[1])) {
                    print(c2,position=c(p[1],0,1,p[4]))
                }
            }
            if (verbose) {
                print("Proliferative:")
                print(pvals[j+length(list)*(i-1),1:3])
                if (!is.null(list2[1])) {
                    print("Invasive:")
                    print(pvals[j+length(list)*(i-1),4:6])
                }
            }
        }
    }
    fdrs <- pvals
    fdrs[1:length(fdrs)] <- p.adjust(as.vector(pvals),method="BH")
    return(list(pvals=pvals,fdrs=fdrs))
}
#' Add logic.
#'
#' extend model with node representing logic gate
#' @export
#' @param child define the child
#' @param logic define the logical gate
#' @param model normal model
#' @examples
#' model <- CreateRandomGraph(c("Ikk1", "Ikk2", "RelA"))
#' model2 <- AddLogicGates("RelA", "OR", model)
#' @return model list with additional logic gate
AddLogicGates <- function(child, logic, model) {
    ## Extends model with a node representing a logic gate for plotting
    parents <- names(which(model[,child]==1))
    redirectEdges <- function(model, parents, child) {
        ## Redirect edges via most recently added logic gate
        model[parents, child] <- 0
        model[nrow(model), child]   <- 1
        model[parents, ncol(model)] <- 1
        return(model)
    }
    model <- cbind(model, rep(0, dim(model)[1]))
    model <- rbind(model, rep(0, dim(model)[2]))
    rownames(model)[dim(model)[1]] <- logic
    colnames(model)[dim(model)[1]] <- logic
    model <- redirectEdges(model, parents, child)
    return(model)
}

#' Plot pathway.
#'
#' Plots the winning pathway structure
#' @param x object of class epiNEM
#' @param ... other arguments
#' @export
#' @method plot epiNEM
#' @importFrom igraph graph.adjacency plot.igraph layout.fruchterman.reingold
#' @examples
#' data <- matrix(sample(c(0,1), 100*4, replace = TRUE), 100, 4)
#' colnames(data) <- c("A", "A.B", "B", "C")
#' rownames(data) <- paste("E", 1:100, sep = "_")
#' res <- epiNEM(data, method = "exhaustive")
#' plot(res)
#' @return plot of the logical network
plot.epiNEM <- function(x, ...) {
    results <- x
    result <- results$origModel
    logics <- results$logics
    ## logics <- paste("NOT ", gsub("masks.*", "", logics), sep = "")
    Egeneset <- results$EGeneset
    diag(result) <- 0
    nGenes    <- ncol(result)
    geneColor <- rep("lightpink", nGenes)
    geneShape <- rep("circle", nGenes)
    EGenes     <- rownames(Egeneset)
    nEGenes    <- nrow(Egeneset)
    EGeneShape <- rep("circle", nEGenes)
    EGeneColor <- rep("lightgrey", nEGenes)
    child <- colnames(result)[results$column]
    for (i in 1:length(logics)){
        result <- AddLogicGates(child[i], logics[i], result)
    }
    for (i in 1:nEGenes){
        result <- cbind(result, rep(0, dim(result)[1]))
        result <- rbind(result, rep(0, dim(result)[2]))
        rownames(result)[dim(result)[1]] <- Egeneset[i]
        colnames(result)[dim(result)[1]] <- Egeneset[i]
        ## if for unconnected node summarizing non-attached E-Genes
        if(i < nEGenes) {
            result[rownames(Egeneset)[i], ncol(result)]   <- 1
        }
    }
    labels     <- colnames(result)
    nLogics    <- length(logics)
    logicColor <- rep("lightgreen", nLogics)
    logicShape <- rep("rectangle", nLogics)
    geneSize   <- rep(30, nGenes)
    logicSize  <- unlist(lapply(logics, function(l) (nchar(l) + 5) * 3))
    EGeneSize  <- rep(30, nEGenes)
    ## lay3 as layout
    g <- graph.adjacency(result)
    plot.igraph(g, vertex.color=c(geneColor, logicColor, EGeneColor),
                vertex.size=c(geneSize, logicSize, EGeneSize),
                vertex.size2=c(rep(30,nGenes), rep(15,nLogics),
                               rep(30, nEGenes)),
                vertex.shape=c(geneShape, logicShape, EGeneShape),
                edge.arrow.size=c(rep(0.3, nGenes+nLogics+nEGenes)),
                vertex.label=labels,
                layout = layout.fruchterman.reingold)
}

#' Plot screen.
#'
#' Plots the sresults of a systematic knock-out screen
#' @param x object of class epiScreen
#' @param global plot global distribution or for each pair (FALSE)
#' @param ind index of pairs to plot
#' @param colorkey if TRUE prints colorkey
#' @param cexGene size of modulator annotation
#' @param off relative distance from the gene names to the respective
#' likelihoods
#' @param cexLegend font size of the legend
#' @param ... other arguments
#' @export
#' @method plot epiScreen
#' @examples
#' data <- matrix(sample(c(0,1), 100*9, replace = TRUE), 100, 9)
#' colnames(data) <- c("A.B", "A.C", "B.C", "A", "B", "C", "D", "E", "G")
#' rownames(data) <- paste("E", 1:100, sep = "_")
#' res <- epiScreen(data)
#' plot(res)
#' plot(res, global = FALSE, ind = 1:3)
#' @return plot(s) of an epiNEM screen analysis
plot.epiScreen <- function(x, global = TRUE, ind = NULL, colorkey = TRUE,
                           cexGene = 1, off = 0.05, cexLegend = 1, ...) {
    doubles <- x$doubles
    if (is.null(ind[1])) { ind <- 1:length(doubles) }
    if (global) {
        distmat <- x$logic
        distmat[which(distmat == 0)] <- 7
        distmat[which(distmat %in% "AND")] <- 1
        distmat[which(distmat %in% "OR")] <- 2
        distmat[which(distmat %in% "XOR")] <- 3
        distmat[which(distmat %in% "NOEPI")] <- 6
        distmat[which(distmat %in% c("UNCON", "NOINFO", "NOINF"))] <- 7
        for (i in 1:ncol(distmat)) {
            genes <- unlist(strsplit(colnames(distmat)[i], "\\."))
            distmat[which(distmat[, i] %in%
                          paste(genes[1], " masks the effect of ", genes[2],
                                sep = "")), i] <- 4
            distmat[which(distmat[, i] %in%
                          paste(genes[2], " masks the effect of ", genes[1],
                                sep = "")), i] <- 5
        }
        distmat <- apply(distmat, c(1,2), as.numeric)
        for (i in 1:ncol(distmat)) {
            distmat[, i] <- rev(sort(distmat[, i]))
        }
        y <- distmat
        distmat <-
            distmat[, order(apply(distmat, 2, function(x) {
                return(sum(x == 1))
            }))]
        y[which(y == 5)] <- 4
        ## if (nrow(y) > 20) {
            rownames(distmat) <- NULL
        ## }
        labeltext <- c("", "no information\n\n\n", "no epistasis\n\n\n",
                       "masking (NOT B)\n\n\n",
                       "masking (NOT A)\n\n\n", "XOR\n\n\n",
                       "OR\n\n\n", "AND\n\n\n")
        if (colorkey) {
            colorkey <- list(space = "right",
                                  labels = rev(labeltext), width = 1,
                             at = seq(1.5,7.5, length.out = 8))
        } else {
            colorkey <- NULL
        }
        HeatmapOP(distmat, Colv = FALSE, Rowv = FALSE,
                  main = "logic gate distribution",
                  sub = "", col = "Paired", breaks =
                                                seq(0.5,7.5, length.out = 8),
                  colorkey = colorkey,
                  xlab = "double knock-outs",
                  ylab = "modulators\n(different order for each pair)", ...)
    } else {
        palette(c("#4444cc", "#77aa77", "#009933",
                  "#ff0000", "#dd8811", "#aa44bb", "#999900"))
        logicmat0 <- x$logic
        llmat0 <- x$ll
        for (i in 1:length(x$doubles)) {
            if (!(i %in% ind)) { next() }
            logicvec <- logicmat0[, i]
            llvec <- llmat0[, i]
            logicvec <- logicvec[order(llvec, decreasing = TRUE)]
            llvec <- llvec[order(llvec, decreasing = TRUE)]
            parents <- unlist(strsplit(doubles[i], "\\."))
            pchvec <- numeric(length(llvec))
            pchvec[which(logicvec %in% "AND")] <- 1
            pchvec[which(logicvec %in% "OR")] <- 2
            pchvec[which(logicvec %in% "XOR")] <- 3
            pchvec[grep(paste("^", parents[1], sep = ""), logicvec)] <- 4
            pchvec[grep(paste("^", parents[2], sep = ""), logicvec)] <- 5
            pchvec[which(logicvec %in% "NOEPI")] <- 6
            pchvec[which(logicvec %in% c("NOINFO", "NOINF"))] <- 7
            logicvec <- logicvec[-which(logicvec %in% "0")]
            pchvec <- pchvec[-which(pchvec == 0)]
            llvec <- llvec[-which(llvec == 0)]
            colvec <- pchvec
            thetop <- sum(!(logicvec %in% c("UNCON", "NOINFO", "NOINF")))
            if (all(is.infinite(llvec) == TRUE)) {
                llvec[1:length(llvec)] <- -1000
                margin <- 100
                donames <- 30
            } else {
                range <- max(llvec[1:thetop]) - min(llvec[1:thetop])
                if (range == 0) {
                    range <- 10
                    margin <- range*0.25
                    offset <- range*off
                    ylim <- c(llvec[1], llvec[1]+10)
                } else {
                    margin <- range*0.25
                    offset <- range*off
                    ylim <- c(min(llvec[1:thetop]),
                              max(llvec[1:thetop])+margin*0.2+offset+margin*3/5)
                }
                offset <- range*off
            }
            mark <- ""
            legendx <- length(llvec[1:thetop])
            p2max <- max(llvec[1:thetop])
            if (p2max == min(llvec[1:thetop])) {
                p2max <- p2max+margin*0.2
            }
            legendtext <- c("AND", "OR", "XOR",
                            paste(tolower(parents[1])," masks ",
                                  tolower(parents[2]), sep = ""),
                            paste(tolower(parents[2]), " masks ",
                                  tolower(parents[1]), sep = ""),
                            "no epistasis")
            if (thetop == 0) { next() }
            plot = plot(llvec[1:thetop], pch = pchvec[1:thetop],
                        col = colvec[1:thetop],
                        ylab = "likelihood", xlab = "ranked single knockouts",
                        ylim = ylim,
                        xlim = c(1, thetop+(thetop/100)),
                        main = tolower(paste(unlist(strsplit(doubles[i],
                                                             "\\.")),
                                             collapse = " and ")))
            text = text((1:thetop)+(thetop/100),
                        llvec[1:thetop]+offset,
                        labels = tolower(names(llvec)[1:thetop]),
                        srt = 45, pos = 3,
                        offset = 0, cex = cexGene, ...)
            mtext = mtext(mark, side = 3, line = 1, outer = FALSE,
                          cex = 4, adj = 0, ...)
            legend = legend("topright",
                ## legendx, p2max,
                legend = legendtext,
                col = 1:6, pch = 1:6, xjust = 1, yjust = 1,
                cex = cexLegend)
        }
    }
}

#' Plot simulations.
#'
#' Plots the simulation results
#' @param x object of class epiSim
#' @param ... other arguments
#' @export
#' @method plot epiSim
#' @examples
#' res <- SimEpiNEM(runs = 1)
#' plot(res)
#' @return plot(s) of an epiNEM simulation analysis
plot.epiSim <- function(x, ...) {

    sens <- sim$sens
    spec <- sim$spec
    sens2 <- sim$sens2
    spec2 <- sim$spec2
    logics <- sim$logics
    time <- sim$time
    noiselvls <- sim$noiselvls

    acc <- (sens + spec)/2

    acc2 <- (sens2 + spec2)/2

    m <- rbind(c(1,1), c(2,2), c(3,4))

    layout(m)

    do <- sim$do

    do2 <- gsub("^b", "B-NEM",
                gsub("^e", "epiNEM",
                     gsub("^n", "NEM", gsub("^p", "PC algorithm",
                                            gsub("^a", "Aracne", sim$do)))))

    colvec <- accframe2 <- timeframe <- accframe <- logicsframe <- NULL

    colvecorg <- c("orange", "blue", "darkgreen", "brown", "darkgrey")

    for (i in 1:dim(spec2)[1]) {

        colvec <- c(colvec, rep(colvecorg[i], dim(spec2)[3]))

        timeframe[[do2[i]]] <- data.frame(time[i,,])

        colnames(timeframe[[do2[i]]]) <- noiselvls

        accframe2[[do2[i]]] <- data.frame(acc2[i,,])

        colnames(accframe2[[do2[i]]]) <- noiselvls

    }

    timeframe <- as.data.frame(timeframe)

    accframe2 <- as.data.frame(accframe2)

    colnames(timeframe) <- colnames(accframe2) <- rep(noiselvls, length(do2))

    boxplot(as.data.frame(timeframe), col = colvec, main = "running time",
            ylab = "seconds")

    abline(v=(1:(length(do)-1)*length(noiselvls) + 0.5), col = "black",
           lty = 6)

    axis(1, 4+(0:(length(do2)-1))*8,
         do2,
         tick = FALSE, pos = -25)

    boxplot(accframe2, col = colvec,
            main = "accuracy of the inferred edges", ylim = c(0,1))

    abline(v=(1:(length(do)-1)*length(noiselvls) + 0.5), col = "black",
           lty = 6)

    axis(1, 4+(0:(length(do2)-1))*8,
         do2,
         tick = FALSE, pos = -0.2)

    if (any(do %in% c("b", "e"))) {

        colvec2 <- unique(colvec)[which(do %in% c("b", "e"))]

        colvec2 <- rep(colvec2, each = length(noiselvls))

        for (i in 1:sum(do %in% c("b", "e"))) {

            accframe[[do2[i]]] <- data.frame(acc[i,,])

            colnames(accframe[[do2[i]]]) <- noiselvls

            logicsframe[[do2[i]]] <- data.frame(logics[i,,])

            colnames(logicsframe[[do2[i]]]) <- noiselvls

        }

        accframe <- as.data.frame(accframe)

        logicsframe <- as.data.frame(logicsframe)

        colnames(logicsframe) <- colnames(accframe) <- rep(noiselvls, 2)

        boxplot(logicsframe, col = colvec2,
                main = "accuracy of the inferred logic gate",
                ylim = c(0,1))

        abline(v=length(noiselvls)+0.5, col = "black", lty = 6)

        axis(1, 4+(0:(sum(do %in% c("b", "e"))-1))*8,
             do2[which(do %in% c("b", "e"))],
             tick = FALSE, pos = -0.2)

        boxplot(accframe, col = colvec2,
                main = "accuracy of the expected data",
                ylim = c(0,1))

        abline(v=length(noiselvls)+0.5, col = "black", lty = 6)

        axis(1, 4+(0:(sum(do %in% c("b", "e"))-1))*8,
             do2[which(do %in% c("b", "e"))],
             tick = FALSE, pos = -0.2)

    }

}

## ----global_options, include=FALSE--------------------------------------------
knitr::opts_chunk$set(message=FALSE)
fig.cap1 <- "Toy example result for a double mutant (A,B) and one modulator
(C)."
fig.cap2 <- "Distribution of inferred logics for each double knock-out."
fig.cap3 <- "Ranked modulators for one double knock-out."
fig.cap4 <- "Perfect binary effects matrix for each logic."
fig.cap5 <- ""
fig.cap6 <- "Global results for the van Wageningen data set. See the help pages
of plot.epiScreen() and HeatmapOP() for additional parameters."
fig.cap7 <- "Results for one double knock-out of the van Wageningen data set.
See ?plot.epiScreen for further parameters."
fig.cap8 <- "Global result for the Sameith data set."
fig.cap9 <- "Example for one knock-out of the Sameith data set."
fig.cap10 <- "Density of the string-db interaction scores (van Wageningen).
Background (turqoise) and inferred by epiNEM (pink)."
fig.cap11 <- "Density of the string-dbinteraction scores (Sameith). Baackground
(turqoise) and inferred by epiNEM (pink)."
fig.cap12 <- "Density plot for graph-based GO similarity score
(van Wageningen)."
fig.cap13 <- "Density plot for graph-based GO similarity score (Sameith)."
fig.cap14 <- "Enrichment of van Wageningen modulators by KEGG pathways.
Colors refer to false discovery rates. NAs are colored in grey."
fig.cap15 <- "Enrichment of Sameith modulators by KEGG pathways. Colors
refer to false discovery rates. NAs are colored in grey."
fig.cap16 <- "Effect reporter KEGG pathway enrichment (van Wageningen).
Colors refer to false discovery rates. NAs are colored in grey."
fig.cap17 <- "Effect reporter KEGG pathway enrichment (Sameith). Colors
refer to false discovery rates. NAs are colored in grey."

## -----------------------------------------------------------------------------
library(epiNEM)

## -----------------------------------------------------------------------------
data <- matrix(sample(c(0,1), 100*4, replace = TRUE), 100, 4)
colnames(data) <- c("A", "A.B", "B", "C")
rownames(data) <- paste("E", 1:100, sep = "_")
print(head(data))
res <- epiNEM(data, method = "exhaustive")

## ----fig.width = 7, fig.height = 7, fig.cap=fig.cap1--------------------------
plot(res)

## -----------------------------------------------------------------------------
data <- matrix(sample(c(0,1), 100*9, replace = TRUE), 100, 9)
colnames(data) <- c("A.B", "A.C", "B.C", "A", "B", "C", "D", "E", "G")
rownames(data) <- paste("E", 1:100, sep = "_")
res <- epiScreen(data)

## ----fig.width = 4, fig.height = 4, fig.cap = fig.cap2------------------------
plot(res)

## ----fig.width = 5, fig.height = 4, fig.cap = fig.cap3------------------------
plot(res, global = FALSE, ind = 1)

## ----warning=FALSE, fig.width = 9, fig.height=5, fig.cap = fig.cap4-----------
epiAnno()

## -----------------------------------------------------------------------------
data(sim)

## ----fig.width = 7, fig.height = 7, fig.cap = "Simulation results."-----------
plot(sim)

## ----fig.width = 8, fig.height = 4, fig.cap = fig.cap6------------------------
data(wagscreen)

doubles <- wagscreen$doubles

dataWag <- wagscreen$dataWag

## clean up the results:

if (length(grep("fus3|ptp2.ptc2", wagscreen$doubles)) > 0) {
    wagscreen$doubles <- wagscreen$doubles[-grep("fus3|ptp2.ptc2",
                                                 wagscreen$doubles)]
    
    wagscreen$dataWag <- wagscreen$dataWag[, -grep("fus3|ptp2.ptc2",
                                                   colnames(wagscreen$dataWag))]
    
    wagscreen$ll <- wagscreen$ll[, -grep("fus3|ptp2.ptc2",
                                         colnames(wagscreen$ll))]
    
    wagscreen$logic <- wagscreen$logic[, -grep("fus3|ptp2.ptc2",
                                               colnames(wagscreen$logic))]
}

plot(wagscreen, xrot = 45, borderwidth = 0)

## ----fig.width = 8, fig.height = 4, fig.cap = fig.cap7------------------------
plot(wagscreen, global = FALSE, ind = 3, cexGene = 0.7, cexLegend = 0.9,
     off = 0.2)

## ----fig.width = 10, fig.height = 6, fig.cap = fig.cap8-----------------------
data(samscreen)

doubles <- samscreen$doubles

dataSam <- samscreen$dataSam

plot(samscreen, xrot = 45, cexCol = 0.6, borderwidth = 0)

## ----fig.width = 8, fig.height = 4, fig.cap = fig.cap9------------------------
plot(samscreen, global = FALSE, ind = 23, cexGene = 0.7, cexLegend = 0.9,
     off = 0.2)

## ----fig.width = 8, fig.height = 5, fig.cap = fig.cap10-----------------------
library(STRINGdb)

## get_STRING_species(version="10", species_name=NULL)[26, ] # 4932

string_db <- STRINGdb$new( version="11", species=4932, score_threshold=0,
                          input_directory="")
data(wageningen_string)

string.scores <- wageningen_string$string.scores
string.names <- wageningen_string$string.names

tmp <- string_db$get_interactions(
    string_db$mp(unique(unlist(strsplit(colnames(dataWag), "\\.")))))

stsc <- unlist(string.scores)

denspval <- wilcox.test(stsc, unlist(tmp$combined_score),
                        alternative = "greater")$p.value

for (i in 100:1) {

    if (denspval < 10^(-i)) {

        denspval <- paste("< ", 10^(-i), sep = "")

    } 

}

plot(density(stsc), col = "#00000000",
     ylim = c(0, max(c(max(density(stsc)$y),
                       max(density(unlist(tmp$combined_score))$y)))),
     main = paste("van Wageningen String-db interaction scores", sep = ""),
     xlab = "",
     cex.main = 1.5)
polygon(density(stsc), col = "#ff000066")

legend("topright", legend=paste("p-value", denspval, " "), cex = 1.5)

mtext = mtext("A", side = 3, line = 1, outer = FALSE, cex = 3, adj = 0,
              at = par("usr")[1] - (par("usr")[2]-par("usr")[1])*0.1)

lines(density(unlist(tmp$combined_score)), col = "#00000000")
polygon(density(unlist(tmp$combined_score)), col = "#00ffff66")

data(sameith_string)

string.scores2 <- sameith_string$string.scores2
string.names2 <- sameith_string$string.names2

tmp <- string_db$get_interactions(
    string_db$mp(unique(unlist(strsplit(colnames(dataSam), "\\.")))))

stsc <- unlist(string.scores2)

denspval <- wilcox.test(stsc, unlist(tmp$combined_score),
                        alternative = "greater")$p.value

for (i in 100:1) {

    if (denspval < 10^(-i)) {

        denspval <- paste("< ", 10^(-i), sep = "")

    } 

}

plot(density(stsc), col = "#00000000",
     ylim = c(0, max(c(max(density(stsc)$y),
                       max(density(unlist(tmp$combined_score))$y)))),
     main = paste("Sameith String-db interaction scores", sp = ""),
     xlab = "",
     cex.main = 1.5)
polygon(density(stsc), col = "#ff000066")

legend("topright", legend=paste("p-value", denspval, " "), cex = 1.5)

mtext = mtext("B", side = 3, line = 1, outer = FALSE, cex = 3, adj = 0,
              at = par("usr")[1] - (par("usr")[2]-par("usr")[1])*0.1)

lines(density(unlist(tmp$combined_score)), col = "#00000000")
polygon(density(unlist(tmp$combined_score)), col = "#00ffff66")

## ----fig.width = 8, fig.height = 5, fig.cap = fig.cap12-----------------------
data(wageningen_GO)

GOepi <- wageningen_GO$epi
GOall <- wageningen_GO$all

denspval <- wilcox.test(GOepi, GOall, alternative = "greater")$p.value

for (i in 100:1) {
    if (i <= 2) {
        for (j in 1:9) {
            if (denspval < j*10^(-i)) {
                denspval <- paste("< ", j*10^(-i), sep = "")
            }
        }
    } else {
        if (denspval < 10^(-i)) {
            denspval <- paste("< ", 10^(-i), sep = "")
        }
    }
}

plot(density(GOepi), col = "#00000000",
     ylim = c(0, max(c(max(density(GOepi)$y),
                       max(density(unlist(GOall))$y)))),
     main = "van Wageningen Go similarity scores",
     xlab = "",
     cex.main = 1.5)
polygon(density(GOepi), col = "#ff000066")

legend("topleft", legend=paste("p-value", denspval, "   "), cex = 1.5)

mtext = mtext("C", side = 3, line = 1, outer = FALSE, cex = 3, adj = 0,
              at = par("usr")[1] - (par("usr")[2]-par("usr")[1])*0.1)

lines(density(unlist(GOall)), col = "#00000000")
polygon(density(unlist(GOall)), col = "#00ffff66")

## ----fig.width = 8, fig.height = 5, fig.cap = fig.cap13-----------------------
data(sameith_GO)

GOepi2 <- sameith_GO$epi
GOall2 <- sameith_GO$all

denspval <- wilcox.test(GOepi2, GOall2, alternative = "greater")$p.value

for (i in 100:1) {
    if (i <= 2) {
        for (j in 1:9) {
            if (denspval < j*10^(-i)) {
                denspval <- paste("< ", j*10^(-i), sep = "")
            }
        }
    } else {
        if (denspval < 10^(-i)) {
            denspval <- paste("< ", 10^(-i), sep = "")
        }
    }
}

plot(density(GOepi2), col = "#00000000",
     ylim = c(0, max(c(max(density(GOepi2)$y),
                       max(density(unlist(GOall2))$y)))),
     main = "Sameith Go similarity scores",
     xlab = "",
     cex.main = 1.5)
polygon(density(GOepi2), col = "#ff000066")

legend("topleft", legend=paste("p-value", denspval, "   "), cex = 1.5)

mtext = mtext("D", side = 3, line = 1, outer = FALSE, cex = 3, adj = 0,
              at = par("usr")[1] - (par("usr")[2]-par("usr")[1])*0.1)

lines(density(unlist(GOall2)), col = "#00000000")
polygon(density(unlist(GOall2)), col = "#00ffff66")

## ----fig.width = 8, fig.height = 5, fig.cap = fig.cap14-----------------------
data(wageningen_GO)

golist <- wageningen_GO$golist

goterms <- character()

for (i in 1:length(golist)) {

    if (i %in% c(5,8)) { next() }

    goterms <- c(goterms,
                 golist[[i]]$term_description[which(golist[[i]]$pvalue_fdr
                                                    < 1)])

}

gomat <- matrix(NA, length(unique(goterms)), ncol(wagscreen$ll))

rownames(gomat) <- sort(unique(goterms))
colnames(gomat) <- colnames(wagscreen$ll)

for (i in 1:ncol(wagscreen$ll)) {

    gotmp <- golist[[i]]
    gotmp <- gotmp[order(gotmp$term_description), ]

    gomat[which(rownames(gomat) %in% golist[[i]]$term_description), i] <-
        golist[[i]][which(golist[[i]]$term_description %in% rownames(gomat)), 4]

}

if (nrow(gomat) > 20) {
    rownames(gomat) <- NULL
}

HeatmapOP(gomat,
          bordercol = "transparent",
          main = "", sub = "",
          xrot = 45, col = "RdYlBu", breaks = 100)

## ----fig.width = 12, fig.height = 3, fig.cap = fig.cap15----------------------
data(sameith_GO)

golist2 <- sameith_GO$golist

goterms <- character()

for (i in 1:length(golist2)) {

    goterms <-
        c(goterms,
          golist2[[i]]$term_description[which(golist2[[i]]$pvalue_fdr < 0.1)])

}

gomat <- matrix(NA, length(unique(goterms)), ncol(samscreen$ll))

rownames(gomat) <- sort(unique(goterms))
colnames(gomat) <- colnames(samscreen$ll)

for (i in 1:ncol(samscreen$ll)) {

    gotmp <- golist2[[i]]
    gotmp <- gotmp[order(gotmp$term_description), ]

    gomat[which(rownames(gomat) %in% golist2[[i]]$term_description), i] <-
        golist2[[i]][which(golist2[[i]]$term_description %in%
                           rownames(gomat)), 4]

}

if (nrow(gomat) > 20) {
    rownames(gomat) <- NULL
}

colnames(gomat) <- tolower(colnames(gomat))

HeatmapOP(gomat,
          bordercol = "transparent",
          main = "", sub = "",
          xrot = 45, cexCol = 0.5, col = "RdYlBu", breaks = 100)

## ----fig.height = 7, fig.width = 14, fig.cap = fig.cap16----------------------
gos <- unique(wageningen_GO$gos)

egenego <- wageningen_GO$egenego

gomat <- array(NA, c(length(gos), nrow(wagscreen$ll), ncol(wagscreen$ll)))

rownames(gomat) <- sort(gos)

colnames(gomat) <- rownames(wagscreen$ll)

dimnames(gomat)[[3]] <- colnames(wagscreen$ll)

for (i in 1:length(wagscreen$targets)) {
    if (length(wagscreen$targets[[i]]) == 0) { next() }
    for (j in 1:length(wagscreen$targets[[i]])) {
        if (dim(egenego[[i]][[j]])[1] > 0) {
            gomat[which(rownames(gomat) %in%
                        egenego[[i]][[j]]$term_description),
                  which(dimnames(gomat)[[2]] %in%
                        names(wagscreen$targets[[i]])[j]), i] <-
                egenego[[i]][[j]]$pvalue_fdr[
                                     order(egenego[[i]][[j]]$term_description)]
        }
    }
}

gomat <- apply(gomat, c(1,2), mean, na.rm = TRUE)

gomat <- gomat[order(apply(gomat, 1, function(x)
    return(sum(is.na(x) == FALSE)))), ]

gomat <- gomat[, rev(order(apply(gomat, 2, function(x)
    return(sum(is.na(x) == FALSE)))))]

gomat <- gomat[, which(apply(gomat, 2,
                             function(x) return(any(is.na(x) == FALSE))))]

HeatmapOP(gomat, xrot = 45, Colv = FALSE, Rowv = FALSE,
          col = "RdYlBu", main = "", sub = "", breaks = 100)

## ----fig.height = 9, fig.width = 16, fig.cap = fig.cap17----------------------
gos2 <- unique(sameith_GO$gos)

egenego2 <- sameith_GO$egenego

gomat <- array(NA, c(length(gos2), nrow(samscreen$ll), ncol(samscreen$ll)))

rownames(gomat) <- sort(gos2)

colnames(gomat) <- rownames(samscreen$ll)

dimnames(gomat)[[3]] <- colnames(samscreen$ll)

for (i in 1:length(samscreen$targets)) {
    if (length(samscreen$targets[[i]]) == 0) { next() }
    for (j in 1:length(samscreen$targets[[i]])) {
        if (dim(egenego2[[i]][[j]])[1] > 0) {
            gomat[which(rownames(gomat) %in%
                        egenego2[[i]][[j]]$term_description),
                  which(dimnames(gomat)[[2]] %in%
                        names(samscreen$targets[[i]])[j]), i] <-
                egenego2[[i]][[j]]$pvalue_fdr[
                                      order(
                                          egenego2[[i]][[j]]$term_description)]
        }
    }
}

gomat <- apply(gomat, c(1,2), mean, na.rm = TRUE)

gomat <- gomat[order(apply(gomat, 1, function(x)
    return(sum(is.na(x) == FALSE)))), ]

gomat <- gomat[, rev(order(apply(gomat, 2, function(x)
    return(sum(is.na(x) == FALSE)))))]

gomat <- gomat[, which(apply(gomat, 2,
                             function(x) return(any(is.na(x) == FALSE))))]

colnames(gomat) <- tolower(colnames(gomat))

HeatmapOP(gomat, xrot = 45, Colv = FALSE, Rowv = FALSE,
          col = "RdYlBu", main = "", sub = "", breaks = 100)

## ----eval=FALSE---------------------------------------------------------------
# ###### simulation:
# 
# ## install_github("MartinFXP/B-NEM"); library(bnem)
# 
# library(nem)
# 
# library(minet)
# 
# library(pcalg)
# 
# runs <- 100
# 
# noiselvls <- c(0.01, 0.025, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5)
# 
# random <- list(FPrate = 0.1, FNrate = noiselvls,
#                single = 4, double = 1, reporters = 100, replicates = 3)
# 
# do <- c("n", "p", "a", "e", "b")
# 
# maxTime <- FALSE
# 
# forcelogic <- TRUE
# 
# epinemsearch <- "greedy"
# 
# nIterations <- 3
# 
# bnemsearch <- "genetic"
# 
# simresults <- SimEpiNEM(runs, do, random, maxTime, forcelogic,
# epinemsearch, bnemsearch, nIterations)
# 
# sim <- simresults
# 
# ###### yeast van Wageningen et al.:
# 
# file <- paste("http://www.holstegelab.nl/",
#               "publications/sv/signaling_redundancy/downloads/DataS1.txt",
#               sep = "")
# 
# data <- read.delim(file)
# 
# dataM <- data[-(1), (1+(1:(324/2))*2)]
# 
# dataP <- data[-(1), (2+(1:(324/2))*2)]
# 
# data[, 2] <- as.character(data[, 2])
# 
# rndup <- which(duplicated(data[, 2]) == TRUE)
# 
# data[rndup, 2] <- paste(data[rndup, 2], "_dup", sep = "")
# 
# rownames(dataM) <- rownames(dataP) <- data[2:nrow(data), 2]
# 
# dataM <- dataM[-1, ]
# 
# dataP <- dataP[-1, ]
# 
# dataM <- apply(dataM, c(1,2), as.numeric)
# 
# dataP <- apply(dataP, c(1,2), as.numeric)
# 
# dataBin <- dataM
# 
# sig <- 0.05
# 
# cutoff <- log2(1.7)
# 
# dataBin[which(dataP < sig & dataP > 0 & abs(dataM) >= cutoff)] <- 1
# 
# dataBin[which(dataP >= sig | dataP == 0 | abs(dataM) < cutoff)] <- 0
# 
# dataBin <- dataBin[-which(apply(dataBin, 1, max) == 0), ]
# 
# dataBinWag <- dataBin
# 
# colnames(dataBin) <- gsub(".del.vs..wt", "", colnames(dataBin))
# 
# colnames(dataBin) <- gsub(".del", "", colnames(dataBin))
# 
# doubles <- colnames(dataBin)[grep("\\.", colnames(dataBin))]
# 
# if (length(grep("vs", doubles)) > 0) {
#     doubles <- sort(doubles[-grep("vs", doubles)])
# } else { doubles <- sort(doubles) }
# 
# doubles.genes <- unique(unlist(strsplit(doubles, "\\.")))
# 
# if (length(grep("\\.", colnames(dataBin))) > 0) {
#     singles <- colnames(dataBin)[-grep("\\.", colnames(dataBin))]
# } else { singles <- sort(singles) }
# 
# singles <- unique(sort(singles))
# 
# wagscreen <- epiScreen(dataBin[, -grep("fus3\\.|ptp2.ptc2", colnames(dataBin))])
# 
# wagscreen$dataWag <- dataBin[, -grep("fus3.|ptp2.ptc2", colnames(dataBin))]
# 
# ###### yeast Sameith et al.:
# 
# file <- paste("http://www.holstegelab.nl/",
#               "publications/GSTF_geneticinteractions/",
#               "downloads/del_mutants_limma.txt", sep = "")
# 
# data <- read.delim(file)
# 
# data <- apply(data, c(1,2), as.character)
# 
# dataM <- data[-1, which(data[1, ] %in% "M")]
# 
# dataM <- apply(dataM, c(1,2), as.numeric)
# 
# dataP <- data[-1, which(data[1, ] %in% "p.value")]
# 
# dataP <- apply(dataP, c(1,2), as.numeric)
# 
# rownames(dataM) <- rownames(dataP) <- data[2:nrow(data), 1]
# 
# dataBin <- dataM
# 
# sig <- 0.01
# 
# cutoff <- log2(1.5)
# 
# dataBin[which(dataP < sig & dataP > 0 & abs(dataM) >= cutoff)] <- 1
# 
# dataBin[which(dataP >= sig | dataP == 0 | abs(dataM) < cutoff)] <- 0
# 
# dataBin <- dataBin[-which(apply(dataBin, 1, max) == 0), ]
# 
# colnames(dataBin) <- gsub("\\.\\.\\.", "\\.", colnames(dataBin))
# 
# doubles <- colnames(dataBin)[grep("\\.", colnames(dataBin))]
# 
# if (length(grep("vs", doubles)) > 0) {
#     doubles <- sort(doubles[-grep("vs", doubles)])
# } else { doubles <- sort(doubles) }
# 
# doubles.genes <- unique(unlist(strsplit(doubles, "\\.")))
# 
# if (length(grep("\\.", colnames(dataBin))) > 0) {
#     singles <- colnames(dataBin)[-grep("\\.", colnames(dataBin))]
# } else { singles <- sort(singles) }
# 
# singles <- unique(sort(singles))
# 
# samscreen <- epiScreen(dataBin)
# 
# samscreen$dataSam <- dataBin
# 
# ## String-db interaction scores:
# 
# library(STRINGdb)
# 
# get_STRING_species(version="10", species_name=NULL)[26, ] # 4932
# 
# string_db <- STRINGdb$new( version="10", species=4932, score_threshold=0,
#                           input_directory="")
# 
# llmat <- wagscreen$ll
# 
# logicmat <- wagscreen$logic
# 
# string.scores <- list()
# 
# string.names <- character()
# 
# for (i in 1:ncol(llmat)) {
#     if (sum(!(llmat[, i] %in% c(0,-Inf))) > 0) {
#         top30 <- llmat[, i]
#         top30[which(top30 == 0)] <- -Inf
#         top30 <- top30[which(!(llmat[, i] %in% c(0,-Inf)))]
#         top30 <- top30[order(top30,decreasing = TRUE)[1:min(30, sum(!(llmat[, i]
#             %in% c(0,-Inf))))]]
# 
#         doubles <- unlist(strsplit(colnames(llmat)[i], "\\."))
# 
#         for (j in names(top30)) {
#             tmp <- string_db$get_interactions(string_db$mp(c(doubles[1], j)))
#             string.scores <- c(string.scores, tmp$combined_score)
#             string.names <- c(string.names, paste(sort(c(doubles[1], j)),
#                                                   collapse = "_"))
#             tmp <- string_db$get_interactions(string_db$mp(c(doubles[2], j)))
#             string.scores <- c(string.scores, tmp$combined_score)
#             string.names <- c(string.names, paste(sort(c(doubles[2], j)),
#                                                   collapse = "_"))
#         }
# 
#     } else {
#         next()
#     }
# }
# 
# 
# llmat <- samscreen$ll
# 
# logicmat <- samscreen$logic
# 
# string.scores2 <- list()
# 
# string.names2 <- character()
# 
# for (i in 1:ncol(llmat)) {
# 
#     if (sum(!(llmat[, i] %in% c(0,-Inf))) > 0) {
#         top30 <- llmat[, i]
#         top30[which(top30 == 0)] <- -Inf
#         top30 <- top30[which(!(llmat[, i] %in% c(0,-Inf)))]
#         top30 <- top30[order(top30, decreasing = TRUE)
#                        [1:min(30, sum(!(llmat[, i] %in% c(0,-Inf))))]]
# 
#         doubles <- unlist(strsplit(colnames(llmat)[i], "\\."))
# 
#         for (j in names(top30)) {
#             tmp <- string_db$get_interactions(string_db$mp(c(doubles[1], j)))
#             string.scores2 <- c(string.scores2, tmp$combined_score)
#             string.names2 <- c(string.names2, paste(sort(c(doubles[1], j)),
#                                                     collapse = "_"))
#             tmp <- string_db$get_interactions(string_db$mp(c(doubles[2], j)))
#             string.scores2 <- c(string.scores2, tmp$combined_score)
#             string.names2 <- c(string.names2, paste(sort(c(doubles[2], j)),
#                                                     collapse = "_"))
#         }
# 
#     } else {
#         next()
#     }
# 
# }
# 
# ## graph based GO similarity scores:
# 
# library(GOSemSim)
# library(AnnotationHub)
# library(org.Sc.sgd.db)
# 
# ystGO <- godata("org.Sc.sgd.db", ont = "BP",
#                 keytype = keytypes(org.Sc.sgd.db)[11], computeIC = FALSE)
# 
# ## van Wageningen et al.:
# 
# GOepi <- numeric()
# 
# for (i in 1:ncol(wagscreen$ll)) {
#     if (i %in% grep("fus3|ptp2.ptc2", colnames(wagscreen$ll))) { next() }
#     pair <- toupper(unlist(strsplit(colnames(wagscreen$ll)[i], "\\.")))
#     for (j in which(!is.infinite(wagscreen$ll[, i]) == TRUE &
#                     wagscreen$ll[, i] != 0)) {
#         tmp <- clusterSim(pair, toupper(rownames(wagscreen$ll)[j]),
#                        semData = ystGO, combine = "max")
#         if (!is.na(tmp[1])) {
#             GOepi <- c(GOepi, tmp)
#         }
#     }
# }
# 
# GOall <- numeric()
# 
# for (i in colnames(wagscreen$ll)) {
#     pair <- toupper(unlist(strsplit(i, "\\.")))
#     for (j in rownames(wagscreen$ll)) {
#         tmp <- clusterSim(pair, toupper(j), semData = ystGO, combine = "max")
#         if (!is.na(tmp[1])) {
#             GOall <- c(GOall, tmp)
#         }
#     }
# }
# 
# ## Sameith et al.:
# 
# GOepi2 <- numeric()
# 
# for (i in 1:ncol(samscreen$ll)) {
#     if (i %in% grep("fus3|ptp2.ptc2", colnames(samscreen$ll))) { next() }
#     pair <- toupper(unlist(strsplit(colnames(samscreen$ll)[i], "\\.")))
#     for (j in which(!is.infinite(samscreen$ll[, i]) == TRUE &
#                     samscreen$ll[, i] != 0)) {
#         tmp <- clusterSim(pair, toupper(rownames(samscreen$ll)[j]),
#                        semData = ystGO, combine = "max")
#         if (!is.na(tmp[1])) {
#             GOepi2 <- c(GOepi2, tmp)
#         }
#     }
# }
# 
# GOall2 <- numeric()
# 
# for (i in colnames(samscreen$ll)) {
#     pair <- toupper(unlist(strsplit(i, "\\.")))
#     for (j in rownames(samscreen$ll)) {
#         tmp <- clusterSim(pair, toupper(j), semData = ystGO, combine = "max")
#         if (!is.na(tmp[1])) {
#             GOall2 <- c(GOall2, tmp)
#         }
#     }
# }
# 
# ###### Go enrichment analysis:
# 
# ## van Wageningen et al.:
# 
# string_db$set_background(
#     string_db$mp(unique(c(unlist(strsplit(colnames(wagscreen$ll), "\\.")),
#                           rownames(wagscreen$ll)))))
# 
# golist <- list()
# 
# for (i in 1:ncol(wagscreen$ll)) {
#     golist[[i]] <- string_db$get_enrichment(string_db$mp(unique(
#         c(unlist(strsplit(colnames(wagscreen$ll)[i], "\\.")),
#           rownames(wagscreen$ll)[which(!(wagscreen$logic[, i] %in%
#                                          c("NOINFO", "NOEPI")))]))),
#         category = "KEGG", methodMT = "fdr", iea = TRUE)
# }
# 
# string_db$set_background(string_db$mp(rownames(wagscreen$dataWag)))
# 
# egenego <- list()
# 
# gos <- character()
# 
# for (i in 1:length(wagscreen$targets)) {
#     egenego[[i]] <- list()
#     if (length(wagscreen$targets[[i]]) == 0) { next() }
#     for (j in 1:length(wagscreen$targets[[i]])) {
#         egenego[[i]][[j]] <- string_db$get_enrichment(
#             string_db$mp(wagscreen$targets[[i]][[j]]),
#             category = "KEGG", methodMT = "fdr", iea = TRUE)
#         if (dim(egenego[[i]][[j]])[1] > 0) {
#             gos <- c(gos, egenego[[i]][[j]]$term_description)
#         }
#     }
# }
# ## Sameith et al.:
# 
# string_db$set_background(string_db$mp(unique(c(unlist(
#     strsplit(colnames(samscreen$ll), "\\.")), rownames(samscreen$ll)))))
# 
# golist2 <- list()
# 
# for (i in 1:ncol(samscreen$ll)) {
#     golist2[[i]] <- string_db$get_enrichment(string_db$mp(
#         unique(c(unlist(strsplit(colnames(samscreen$ll)[i], "\\.")),
#                  rownames(samscreen$ll)
#                  [which(!(samscreen$logic[, i] %in% c("NOINFO", "NOEPI")))]))),
#         category = "KEGG", methodMT = "fdr", iea = TRUE)
# }
# 
# string_db$set_background(string_db$mp(rownames(samscreen$dataWag)))
# 
# egenego2 <- list()
# 
# gos2 <- character()
# 
# for (i in 1:length(samscreen$targets)) {
#     egenego2[[i]] <- list()
#     if (length(samscreen$targets[[i]]) == 0) { next() }
#     for (j in 1:length(samscreen$targets[[i]])) {
#         egenego2[[i]][[j]] <- string_db$get_enrichment(
#             string_db$mp(samscreen$targets[[i]][[j]]),
#             category = "KEGG", methodMT = "fdr", iea = TRUE)
#         if (dim(egenego2[[i]][[j]])[1] > 0) {
#             gos2 <- c(gos2, egenego2[[i]][[j]]$term_description)
#         }
#     }
# }

## -----------------------------------------------------------------------------
sessionInfo()

#' Example data: simulation results
#'
#' Contains simulation results. How they were
#' aquired is explained in the vignette.
#' The data conists of a list of data matrices holding
#' sensitivity and specificity
#' (spec, sens) of network edges for the variious methods compared to
#' the ground truth, sensitivity and specificity (sens2, spec2)
#' of the expected data for epiNEM and Boolean NEMs and accuracy
#' of the inferred logics for both. The different methods are in the
#' rows and the columns denote the different independent simulation runs.
#' @docType data
#' @examples
#' data(sim)
#' @name sim
NA

#' Example data: epiNEM results for
#' the Sameith et al., 2015 knock-out screen
#'
#' The result of the epiNEM analysis of the data from
#' "http://www.holstegelab.nl/publications/
#' sv/signaling_redundancy/downloads/DataS1.txt".
#' The data consists of a list of matrices with the likelihoods (ll)
#' for each analysed triple of signalling genes and the inferred logic
#' (logic) for each triple. The signalling genes or modulators C are the
#' rows and the signalling genes from the double knock-downs are in the columns.
#' For details see the vignette.
#' @docType data
#' @examples
#' data(samscreen)
#' @name samscreen
NA

#' Example data: epiNEM results for
#' the Wageningen et al., 2010 knock-out screen
#' "http://www.holstegelab.nl/publications/GSTF_geneticinteractions/
#' downloads/del_mutants_limma.txt"
#'
#' The data consists of a list of matrices with the likelihoods (ll)
#' for each analysed triple of signalling genes and the inferred logic
#' (logic) for each triple. The signalling genes or modulators C are the
#' rows and the signalling genes from the double knock-downs are in the columns.
#' For details see the vignette.
#' @docType data
#' @examples
#' data(wagscreen)
#' @name wagscreen
NA

#' sig. of string interaction scores
#' for Sameith et al., 2015 data
#'
#' The data consists of a list including a vectors of pairs (for interactions)
#' and a corresponding list of interaction scores derived form the
#' string database.
#' For details see the vignette.
#' @docType data
#' @examples
#' data(sameith_string)
#' @name sameith_string
NA

#' sig. of string interaction scores
#' for van Wageningen et al., 2010 data
#'
#' The data consists of a list including a vectors of pairs (for interactions)
#' and a corresponding list of interaction scores derived form the
#' string database.
#' For details see the vignette.
#' @docType data
#' @examples
#' data(wageningen_string)
#' @name wageningen_string
NA

#' graph-based GO similarity scores, string GO annotations
#' for Sameith et al., 2015 data
#'
#' The data consists of lists including epiNEM identified and
#' general similarity scores and GO annotations for each triple.
#' For details see the vignette.
#' @docType data
#' @examples
#' data(sameith_GO)
#' @name sameith_GO
NA

#' graph-based GO similarity scores, string GO annotations
#' for van Wageningen et al., 2015 data
#'
#' The data consists of lists including epiNEM identified and
#' general similarity scores and GO annotations for each triple.
#' For details see the vignette.
#' @docType data
#' @examples
#' data(wageningen_GO)
#' @name wageningen_GO
NA

###--- MAIN SCRIPT ---###
#' Epistatic NEMs - main function.
#'
#' This function contains the inference
#' algorithm to learn logical networks from knock-down data including
#' double knock-downs.
#' @param filename A binary, tab-delimited matrix.
#' Columns: single and double knockdowns.
#' Rows: genes showing effect or not?
#' Default: random; artificial data is
#' generated to 'random' specifications
#' @param method greedy or exhaustive search.
#' Default: greedy
#' @param nIterations number of iterations.
#' Default: 10
#' @param nModels number of Models. Default: 0
#' @param random list specifying how the
#' data should be generated:
#' no. of single mutants, no. of
#' double mutants, no. of reporterGenes,
#' FP-rate, FN-rate, no. of replicates
#' @param ltype likelihood either
#' "marginal" or "maximum"
#' @param para false positive and
#' false negative rates
#' @param init adjacency matrix to initialise the greedy search
#' @author Madeline Diekmann
#' @seealso nem
#' @export
#' @import
#' stats
#' e1071
#' utils
#' @importFrom mnem mnem
#' @return List object with an adjacency matrix denoting the network,
#' the model of the silencing scheme (rows are knock-downs, columns
#' are signalling genes), a string with the inferred logial gates,
#' a column indices denoting position of logical gates, the log transformed
#' likelihood and the effect reporter distribution (rows are the signalling
#' genes including the null node).
#' @examples
#' data <- matrix(sample(c(0,1), 100*4, replace = TRUE), 100, 4)
#' colnames(data) <- c("A", "A.B", "B", "C")
#' rownames(data) <- paste("E", 1:100, sep = "_")
#' res <- epiNEM(data, method = "exhaustive")
#' plot(res)
epiNEM <- function(filename="random",
                   method="greedy",
                   nIterations=10,
                   nModels=0,
                   random=list(single=4,double=1,reporters=100,
                               FPrate=0.1,FNrate=0.1,replicates=1),
                   ltype = "marginal",
                   para = c(0.13, 0.05),
                   init = NULL) {

    ##--- Sanity checks ---#
    methods <- c("greedy", "exhaustive")
    if (!method %in% methods)
        stop("Please enter a valid method: 'greedy' or 'exhaustive'.\n")

    if ((method == "greedy") && (nIterations < 1))
        stop("Please enter a positive number of iterations\n")

    ##--- Process input ---#
    if (is.character(filename)){
        if (filename == "random") {
            ## Generate artificial data
            topology    <- CreateTopology(random$single, random$double)
            topology    <- unlist(topology, recursive=FALSE)
            extTopology <- ExtendTopology(topology$model, random$reporters)
            sortedData <-
                GenerateData(topology$model, extTopology, random$FPrate,
                             random$FNrate, random$replicates)

            mutants   <- rownames(topology$model)
            singleKOs <- sort(colnames(topology$model))
            experiments <- singleKOs
            doubleKOs <- mutants[!mutants %in% singleKOs]
        } else {
            ## Use data from file
            ## data        <- read.table(filename)
            ## sortedData  <- data[, order(unlist(sortedKOs))]
        }
    } else sortedData = filename

    mutants <- colnames(sortedData)
    singleKOs <- sort(unique(unlist(strsplit(mutants, '[.]'))))
    experiments <- singleKOs
    doubleKOs <- mutants[!mutants %in% singleKOs]
    ##-- Initialize data matrices (this takes care of replicate experiments) --
    D1 <- sapply(mutants, function(s)
        rowSums(sortedData[, colnames(sortedData) == s, drop=FALSE]))
    D0 <- sapply(mutants, function(s) sum(colnames(sortedData) == s)) - D1

    ##--- Optimize models ---#
    if (method == "exhaustive" & length(singleKOs) > 5) {
        cat("\nSorry, too many nodes for exhaustive search,\n")
        cat("doing greedy hill climbing instead.\n")
        method = "greedy"
    } else if (method == "exhaustive" & length(singleKOs) == 5) {
        cat("\nThis is going to take a while...\n")
    }
    if (length(doubleKOs) == 0) {
        print("No double perturbations available --> computing NEM")
        return(mnem::mnem(sortedData, search=inference, fpfn=para, k=1))
    } else {
        if (method == "greedy") {
            print(paste("Progress (of ", nIterations, " iterations):",
                        sep = ''))
            iterations <- sapply(1:nIterations, GreedyHillClimber,
                                 experiments, D1, D0, mutants, init)

            index <- which.max(iterations["maxLlh",])
            reconstruct <- includeLogic(iterations[,index]$model,
                                        experiments, mutants)
            reconstruct <- unlist(reconstruct, recursive=FALSE)
            score <- sapply(reconstruct, Mll, D1, D0, ltype, para)
            mll <- unlist(score["mLL",])
            index <- which.max(mll)
            posterior <- score[,index]$posterior

            results <- unlist(reconstruct[index], recursive=FALSE)
            results$score <- mll[index]
            if (ltype %in% "maximum") {
                names(results$score) <- "max"
            } else {
                names(results$score) <- "mLL"
            }
            results$EGeneset <- AttachEGenes(posterior, experiments)
            results$PostScore <- score[,which.max(mll)]$posterior
            cat("\n")
        } else if (method == "exhaustive") {
            basicModels    <- EnumerateModels(length(singleKOs), singleKOs)
            extendedModels <- lapply(basicModels, includeLogic,
                                     experiments, unique(mutants))
            extendedModels <- unlist(unlist(extendedModels, recursive=FALSE),
                                     recursive = FALSE)
            uniqueModels   <- unique(lapply(extendedModels, function(e)
                identity((e["model"]))))
            uniqueModels <- uniqueModels#[1:length(uniqueModels)-1]

            score <- sapply(uniqueModels, Mll, D1, D0, ltype, para)
            mll  <- unlist(score["mLL",])

            bestModel <- uniqueModels[[which.max(mll)]]$model
            allModels <- lapply(1:length(extendedModels), function(i)
                identity(extendedModels[[i]]$model))
            isBest    <- lapply(allModels, IsBestModel, bestModel)
            results   <- extendedModels[isBest == TRUE]
            results <- lapply(results, utils::modifyList,
                              list(score=mll[which.max(mll)]))
            results <- results[[1]]
            if (ltype %in% "maximum") {
                names(results$score) <- "max"
            } else {
                names(results$score) <- "mLL"
            }
            savescore <- score[,which.max(mll)]$posterior
            posterior <- AttachEGenes(score[,which.max(mll)]$posterior,
                                      experiments)
            results$EGeneset <- posterior
            results$PostScore <- savescore

        }

        class(results) <- "epiNEM"

        return(results)
    }
}

###--- HELPER FUNCTIONS ---###

#' Extending topology of normal "nem"
#' @param topology model of a topology from CreateTopology
#' @param nReporters number of effects reporters
#' @author Madeline Diekmann
#' @seealso CreateTopology
#' @export
#' @examples
#' topology <- CreateTopology(3, 1, force = TRUE)
#' topology <- unlist(unique(topology), recursive = FALSE)
#' extTopology <- ExtendTopology(topology$model, 100)
#' @return extended topology in which reporters
#' are linked to pathway genes
ExtendTopology <- function(topology, nReporters) {
    reporters     <- unlist(lapply(1:nReporters,
                                   function(n) paste("reporter", n, sep="-")))
    linkedEffects <- sample(1:ncol(topology), nReporters, replace=TRUE)
    extTopology   <-
        sapply(linkedEffects,
               function(e) lapply(1:ncol(topology),
                                  function(c) ifelse(e == c, 1, 0)))
    extTopology <-
        matrix(unlist(extTopology), ncol=ncol(topology), byrow=TRUE)
    dimnames(extTopology) <- list(reporters, colnames(topology))
    return(extTopology)
}

#' Generate data from extended model.
#'
#' Given a model created from
#' CreateTopology and ExtendTopology, this function creeates acorresponding
#' artificial data matrix, which is used as a ground truth for simulation
#' studies.
#' @param model model of a topology
#' from CreateTopology
#' @param extTopology extended topology
#' @param FPrate false positive rate
#' @param FNrate false negative rate
#' @param replicates number of replicates
#' @author Madeline Diekmann
#' @seealso CreateTopology
#' @export
#' @examples
#' topology <-
#' CreateTopology(3, 1, force = TRUE)
#' topology <-
#' unlist(unique(topology), recursive = FALSE)
#' extTopology <-
#' ExtendTopology(topology$model, 100)
#' sortedData <-
#' GenerateData(topology$model, extTopology, 0.05, 0.13, 3)
#' @return data matrix with effect reporters as rows and knock-downs
#' (including double kock-downs) as columns.
GenerateData <- function(model, extTopology, FPrate, FNrate, replicates) {
    perfectData <- extTopology %*% t(model)
    perfectData <- perfectData[, rep(seq_len(ncol(perfectData)), replicates)]
    fps <- sample(which(perfectData == 0), floor(sum(perfectData == 0)*FPrate))
    fns <- sample(which(perfectData == 1), floor(sum(perfectData == 1)*FNrate))
    noisyData   <- perfectData
    noisyData[c(fps, fns)] <- 1 - noisyData[c(fps, fns)]

    return(noisyData)
}

#' @noRd
IsBestModel <- function(thisModel, bestModel) {
    if (any(dim(thisModel) != dim(bestModel))) return(FALSE)
    else if (any(thisModel != bestModel)) return(FALSE)
    return(TRUE)
}

#' Compare algorithms.
#'
#' Compares different network reconstruction algorithm on
#' simulated data.
#' @param runs number simulation runs
#' @param do string vector of algorithms to
#' compare: e (epiNEM), n (Nested Effects Models),
#' b (B-NEM), p (PC algorithm), a (Aracne), e.g. c("e", "n", "p")
#' @param random list of false poitive rate FPrate, false negative rates
#' FNrate, number of single knock-downs single, number of double
#' knock-downs double, number of effect reporters reporters and number
#' of replicates replicates
#' @param maxTime TRUE if the algorithms are bound to a maximum running
#' time in respect to epiNEM
#' @param forcelogic if TRUE the randomly sampled ground truth network
#' includes a complex logic with probability 1
#' @param epinemsearch greedy or exhaustive search for epiNEM
#' @param bnemsearch genetic or greedy search for B-NEM
#' @param ... additional parameters
#' @author Martin Pirkl
#' @return returns list of specificity and sensitivity of inferred edges
#' (spec, sens) and inferred expected data (spec2, sens2) and accuracy
#' of logics (logics) and running time (time)
#' @export
#' @import
#' pcalg
#' minet
#' @importFrom mnem mnem transitive.reduction
#' @importFrom graph adj
#' @examples
#' res <- SimEpiNEM(runs = 1)
SimEpiNEM <- function(runs = 10, do = c("n", "e"),
                      random = list(FPrate = 0.1,
                                    FNrate = c(0.1, 0.5),
                                    single = 3, double = 1,
                                    reporters = 10,
                                    replicates = 2),
                      maxTime = FALSE,
                      forcelogic = TRUE,
                      epinemsearch = "greedy",
                      bnemsearch = "genetic", ...) {

    noiselvls <- random$FNrate

    spec <- sens  <- logics <- array(0, dim = c(2, runs, length(noiselvls)))

    sens2 <- spec2 <- time <- array(0, dim = c(5, runs, length(noiselvls)))

    logicgate <- matrix("", runs, length(noiselvls))

    edgenr <- matrix(0, runs, length(noiselvls))

    for (i in 1:runs) {

        print(paste("run ", i, sep = ""))

        for (j in 1:length(noiselvls)) {

            print(paste("noiselvl ", j, sep = ""))

            topology <- CreateTopology(random$single, random$double,
                                       force = forcelogic)

            topology <- unlist(unique(topology), recursive = FALSE)

            extTopology <- ExtendTopology(topology$model, random$reporters)

            sortedData <- GenerateData(topology$model, extTopology,
                                       random$FPrate, random$FNrate[j],
                                       random$replicates)

            logicgate[i, j] <- paste(topology$logics, collapse = "_")

            edgenr[i, j] <- sum(topology$origModel == 1)

            if ("e" %in% do) {
                print("epiNEM")

                start <- Sys.time()
                TriplModel <- epiNEM(filename = sortedData,
                                     method = epinemsearch,
                                     ...)
                time[1, i, j] <- difftime(Sys.time(), start, units = "secs")
                print(time[1, i, j])

                tp <- sum(topology$model == 1 & TriplModel$model == 1)
                tn <- sum(topology$model == 0 & TriplModel$model == 0)
                fp <- sum(topology$model == 0 & TriplModel$model == 1)
                fn <- sum(topology$model == 1 & TriplModel$model == 0)
                sens[1, i, j] <- tp/(tp+fn)
                spec[1, i, j] <- tn/(tn+fp)
                tp <- sum(topology$origModel == 1 & TriplModel$origModel == 1)
                tn <- sum(topology$origModel == 0 & TriplModel$origModel == 0)
                fp <- sum(topology$origModel == 0 & TriplModel$origModel == 1)
                fn <- sum(topology$origModel == 1 & TriplModel$origModel == 0)
                sens2[1, i, j] <- tp/(tp+fn)
                spec2[1, i, j] <- tn/(tn+fp)
                tp <- 0
                for (k in 1:length(topology$column)) {
                    for (l in 1:length(TriplModel$column)) {
                        if (topology$column[k] == TriplModel$column[l]) {
                            if (topology$logics[k] %in% TriplModel$logics[l]) {
                                tp <- tp + 1
                            }
                        }
                    }
                }
                logics[1, i, j] <- tp/(length(topology$logics) +
                                       length(TriplModel$logics) - tp)
                print(sens[1, i, j])
                print(spec[1, i, j])
                print(sens2[1, i, j])
                print(spec2[1, i, j])
                print(logics[1, i, j])

            }

            if ("b" %in% do) {
                print("B-NEM")

                gtn <- epiNEM2Bg(topology)

                fc <- cbind(Ctrl_vs_S = -1, epiNEM2Bg(sortedData))*(-1)

                bnemnoise <-
                    sample(1:nrow(fc), floor(nrow(fc)*random$FNrate[j]))

                fc[bnemnoise, 1] <- 0

                ers <- t(topology$model)*(-1)
                colnames(ers) <-
                    paste("S_vs_S_", gsub("\\.", "_", colnames(ers)), sep = "")
                ers <- cbind(Ctrl_vs_S = 1, ers)
                ers <- ers[, order(colnames(ers))]

                CNOlist <- dummyCNOlist(stimuli = "S",
                                        inhibitors = LETTERS[1:random$single],
                                        maxStim = 1, maxInhibit = 2,
                                        signals = LETTERS[1:random$single])

                parents <-
                    unique(unlist(
                        strsplit(colnames(sortedData)[
                            grep("\\.", colnames(sortedData))], "\\.")))

                nodes <-
                    unique(colnames(sortedData)[
                        -grep("\\.", colnames(sortedData))])

                child <- nodes[-which(nodes %in% parents)]

                sifMatrix <- NULL
                for (k in LETTERS[1:random$single]) {
                    sifMatrix <- rbind(sifMatrix, c("S", "1", k))
                    ##, c("S", "-1", k))
                    for (l in LETTERS[1:random$single]) {
                        if (k %in% l) { next() }
                        if (k %in% parents) {
                            sifMatrix <-
                                rbind(sifMatrix, c(k, "1", l), c(k, "-1", l))
                        } else {
                            sifMatrix <- rbind(sifMatrix, c(k, "1", l))
                        }
                    }
                }
                randfile <- paste("pkn_", as.numeric(Sys.time()), sep = "")
                write.table(sifMatrix, file = randfile, sep = "\t",
                            row.names = FALSE, col.names = FALSE, quote = FALSE)
                PKN <- readSIF(randfile)
                unlink(randfile)

                model <- preprocessing(CNOlist, PKN)

                initBstring <- absorption(rep(1, length(model$reacID)), model)

                if (maxTime) {
                    maxTime2 <- time[1, i, j]
                } else { maxTime2 <- Inf }

                start <- Sys.time()
                bga <- bnem(search = bnemsearch,
                            fc=fc,
                            CNOlist=CNOlist,
                            model=model,
                            initBstring=initBstring,
                            draw = FALSE,
                            verbose = FALSE,
                            popSize = popSize,
                            maxTime = maxTime2,
                            parallel = parallel
                            )
                time[2, i, j] <- difftime(Sys.time(), start, units = "secs")
                print(time[2, i, j])

                ers2 <-
                    computeFc(CNOlist,
                              t(simulateStatesRecursive(CNOlist,
                                                        model, bga$bString)))
                ers2 <- ers2[, unique(colnames(fc))]
                ers2 <- ers2[, order(colnames(ers2))]

                tp <- sum(ers == -1 & ers2 == -1)
                tn <- sum(ers == 0 & ers2 == 0)
                fn <- sum(ers == -1 & ers2 == 0)
                fp <- sum(ers == 0 & ers2 == -1)
                sens[2, i, j] <- tp/(tp+fn)
                spec[2, i, j] <- tn/(tn+fp)
                gtn2 <- abs(dnf2adj(gtn))
                if (length(grep("S", rownames(gtn2))) > 0) {
                    gtn2 <-
                        gtn2[-grep("S", rownames(gtn2)),
                             -grep("S", colnames(gtn2))]
                }
                gtn2 <- gtn2[order(rownames(gtn2)), order(colnames(gtn2))]
                res <- abs(dnf2adj(bga$graph))
                if (length(grep("S", rownames(res))) > 0) {
                    res <- as.matrix(res[-grep("S", rownames(res)),
                                         -grep("S", colnames(res))])
                }
                if (dim(res)[1] == 1) {
                    colnames(res) <- rownames(res) <- gsub(".*=", "", bga$graph)
                } else {
                    res <- res[order(rownames(res)), order(colnames(res))]
                }
                if (nrow(res) < nrow(gtn2)) {
                    res2 <-
                        rbind(cbind(res,
                                    matrix(0, nrow(res),
                                           nrow(gtn2) - nrow(res))),
                              matrix(0, nrow(gtn2) - nrow(res), ncol(gtn2)))
                    colnames(res2)[(ncol(res)+1):ncol(res2)] <-
                        colnames(gtn2)[which(!(colnames(gtn2)
                            %in% colnames(res)))]
                    rownames(res2)[(nrow(res)+1):nrow(res2)] <-
                        rownames(gtn2)[which(!(rownames(gtn2)
                            %in% rownames(res)))]
                    res2 <- res2[order(rownames(res2)), order(colnames(res2))]
                    res <- res2
                }
                diag(gtn2) <- diag(res) <- 0
                tp <- sum(gtn2 == 1 & res == 1)
                tn <- sum(gtn2 == 0 & res == 0)
                fn <- sum(gtn2 == 1 & res == 0)
                fp <- sum(gtn2 == 0 & res == 1)
                sens2[2, i, j] <- tp/(tp+fn)
                spec2[2, i, j] <- tn/(tn+fp)
                tp <- sum(bga$graph %in% gtn)
                logics[2, i, j] <- tp/(length(gtn) + length(bga$graph) - tp)
                print(sens[2, i, j])
                print(spec[2, i, j])
                print(sens2[2, i, j])
                print(spec2[2, i, j])
                print(logics[2, i, j])

                print(bga$graph)
                print(gtn)

            }

            if (any(c("n", "p", "a") %in% do)) {

                reddata <- sortedData[, -grep("\\.", colnames(sortedData))]
                gtnadj <- topology$origModel
                gtnadj <-
                    gtnadj[order(apply(gtnadj, 1, sum), decreasing = TRUE),
                           order(apply(gtnadj, 2, sum), decreasing = FALSE)]
                gtnadj[lower.tri(gtnadj)] <- gtnadj[upper.tri(gtnadj)]
                gtnadj <- gtnadj[order(rownames(gtnadj)),
                                 order(colnames(gtnadj))]
                eadj <- topology$origModel
                eadj <- eadj[order(rownames(eadj)), order(colnames(eadj))]
                reddata2 <- matrix(0, nrow(reddata)*random$replicates,
                                   length(unique(colnames(reddata))))
                for (k in 1:length(unique(colnames(reddata)))) {
                    reddata2[, k] <-
                        as.vector(reddata[,
                                          which(colnames(reddata) %in%
                                                unique(colnames(reddata))[k])])
                }
                colnames(reddata2) <- unique(colnames(reddata))

            }

            if ("n" %in% do) {
                print("NEM")

                start <- Sys.time()
                if (epinemsearch %in% "greedy") {
                    nemres <- mnem::mnem(reddata, k = 1)
                } else {
                    nemres <- mnem::mnem(reddata, k = 1, search = "exhaustive")
                }
                nadj <- mnem::transitive.reduction(nemres$comp[[1]]$phi)
                time[3, i, j] <- difftime(Sys.time(), start, units = "secs")
                print(time[3, i, j])

                tp <- sum(eadj  == 1 & nadj == 1)
                tn <- sum(eadj == 0 & nadj == 0)
                fp <- sum(eadj == 0 & nadj == 1)
                fn <- sum(eadj == 1 & nadj == 0)
                sens2[3, i, j] <- tp/(tp+fn)
                spec2[3, i, j] <- tn/(tn+fp)
                print(sens2[3, i, j])
                print(spec2[3, i, j])

            }

            if ("p" %in% do) {
                print("PCalg")

                start <- Sys.time()
                pc.fit <- pc(suffStat = list(C = cor(reddata2),
                                             n = nrow(reddata2)),
                             indepTest = gaussCItest,
                             ## indep.test: partial correlations
                             alpha=0.05, labels = colnames(reddata2),
                             verbose = FALSE)
                graph2adj <- function(gR) {
                    adj.matrix <- matrix(0,
                                         length(graph::nodes(gR)),
                                         length(graph::nodes(gR))
                                         )
                    rownames(adj.matrix) <- graph::nodes(gR)
                    colnames(adj.matrix) <- graph::nodes(gR)
                    for (i in 1:length(nodes(gR))) {
                        adj.matrix[graph::nodes(gR)[i],
                                   adj(gR,graph::nodes(gR)[i])[[1]]] <- 1
                    }

                    return(adj.matrix)
                }
                pcadj <- graph2adj(pc.fit@graph)
                time[4, i, j] <- difftime(Sys.time(), start, units = "secs")
                print(time[4, i, j])

                tp <- sum(gtnadj == 1 & pcadj == 1)
                tn <- sum(gtnadj  == 0 & pcadj == 0)
                fp <- sum(gtnadj == 0 & pcadj == 1)
                fn <- sum(gtnadj == 1 & pcadj == 0)
                sens2[4, i, j] <- tp/(tp+fn)
                spec2[4, i, j] <- tn/(tn+fp)
                print(sens2[4, i, j])
                print(spec2[4, i, j])

            }

            if ("a" %in% do) {
                print("Aracne")

                start <- Sys.time()
                ares <- build.mim(reddata2)
                ares <- aracne(ares)
                ares[which(ares > 0)] <- 1
                ares[which(ares < 0)] <- -1
                ares <- ares[order(rownames(ares)), order(colnames(ares))]
                nas <- which(is.na(ares) == TRUE)
                ares[nas] <- 0
                diag(ares) <- 0
                time[5, i, j] <- difftime(Sys.time(), start, units = "secs")
                print(time[5, i, j])

                tp <- sum(gtnadj == 1 & ares == 1)
                tn <- sum(gtnadj == 0 & ares == 0)
                fp <- sum(gtnadj == 0 & ares == 1)
                fn <- sum(gtnadj == 1 & ares == 0)
                sens2[5, i, j] <- tp/(tp+fn)
                spec2[5, i, j] <- tn/(tn+fp)
                print(sens2[5, i, j])
                print(spec2[5, i, j])

            }

        }

    }

    result <- list(sens = sens, spec = spec, sens2 = sens2,
                   spec2 = spec2, logics = logics, time = time)

    class(result) <- "epiSim"

    return(result)

}

#' Heatmap.
#'
#' Heatmap function based on the lattice package
#' more information: ?xyplot
#' @param x Matrix.
#' @param col Color. See brewer.pal.info for all available
#' color schemes. Alternatively, any number of colors, which
#'  are then used to create a color gradient. E.g., c('blue','red')
#'   produces a color scheme with a gradient from blue to red.
#' @param colNA color for NAs; defaul is grey
#' @param coln Number of colors.
#' @param bordercol Border color.
#' @param borderwidth Border width.
#' @param breaks Defines the breaks in the color range. "sym"
#' makes the breaks symmetric around 0.
#' @param main Main title.
#' @param sub Subtitle.
#' @param dendrogram Draw dendrogram with "both", "col" or
#' "row", or do not draw with "none".
#' @param colorkey Draw colorkey "left", "right" (default), "top", "bottom"
#' or NULL for no colorkey. See ?lattice::levelplot for more complex colorkey
#' options.
#' @param Colv Cluster columns (TRUE) or not (FALSE).
#' @param Rowv Cluster rows (TRUE) or not (FALSE).
#' @param xrot Rotate the column names by degree.
#' @param yrot Rotate the row names by degree.
#' @param shrink c(x,y) defines a range of size for the data
#' boxes from low to high.
#' @param cexCol Font size of column names.
#' @param cexRow Font size of row names.
#' @param cexMain Font size of main title.
#' @param cexSub Font size of subtitle.
#' @param colSideColors Defines a numeric vector to annotate
#' columns with different colors.
#' @param aspect "iso" for quadratic boxes or "fill" for
#' streched boxes.
#' @param contour TRUE adds a contour plot.
#' @param useRaster TRUE to add raster visuals
#' @param xlab Label for the x-axis.
#' @param ylab Label for the y-axis.
#' @param colSideColorsPos Place colSideColors at the "top" or "bottom".
#' @param clust p, s, or k for correlation clustering
#' @param clusterx Optional data matrix y with the same dimensions
#' as x. x's columns or rows are sorted by the cluster information of y.
#' Col- and rownames of y must be in the same order as in x.
#' @param axis.padding padding around the heatmap (0.5 is no padding,
#' default)
#' @param legend list obejct. For parameters see base function ?legend for details. x and y parameters are relative to the inside of the heatmap and are between 0 and 1. E.g., to place the legend outside of the heatmap x and y need to be either less than 0 or greater than 1.
#' @param \dots Optional arguments.
#' @author Martin Pirkl & Oscar Perpinan
#' at http://oscarperpinan.github.io/rastervis/
#' @return lattice object/matrix
#' @export
#' @import
#' lattice
#' latticeExtra
#' RColorBrewer
#' grDevices
#' @importFrom BoutrosLab.plotting.general legend.grob
#' @examples
#' x <- matrix(rnorm(50), 10, 5)
#' HeatmapOP(x, dendrogram = "both", aspect = "iso", xrot = 45)
HeatmapOP <-
    function(x, col = "RdYlGn", colNA = "grey", coln = 11, bordercol = "grey",
             borderwidth = 0.1, breaks = "sym",
             main = "",
             sub = "",
             dendrogram = "none", colorkey = "right", Colv = TRUE,
             Rowv = TRUE, xrot = 90, yrot = 0, shrink = c(1,1), cexCol = 1,
             cexRow = 1, cexMain = 1, cexSub = 1,
             colSideColors = NULL, aspect = "fill",
             contour = FALSE, useRaster = FALSE, xlab = NULL, ylab = NULL,
             colSideColorsPos = "top", clust = NULL, clusterx = NULL,
             axis.padding = 0.5, legend = NULL, ...) {
        if (!is.null(legend)) {
            legend0 <- list(inside=list(fun=legend.grob,
                                        x=legend$x,y=legend$y,
                                        args=list(legends=list(
                                            legend=list(colours=legend$fill,labels=legend$legend,
                                                        border=legend$border,title=legend$title)
                                        ))))
        } else {
            legend0 <- NULL
        }
        if (!is.null(colorkey)) {
            if (!is.list(colorkey) & colorkey[1] %in% c("left", "right", "top", "bottom")) {
                colorkey <- list(space = colorkey)
            }
        }
        if (nrow(x)==1) {
            Rowv <- FALSE
        }
        if (ncol(x)==1) {
            Colv <- FALSE
        }
        if (max(x, na.rm = TRUE) == min(x, na.rm = TRUE)) {
            x <- matrix(rnorm(100), 10, 10)
            main <- "max value equals min value"
            sub <- "random matrix plotted"
        }
        if (is.null(breaks[1])) {
            breaks <- seq(min(x, na.rm = TRUE),max(x, na.rm = TRUE),
            (max(x, na.rm = TRUE) - min(x, na.rm = TRUE))/45)
        }
        if ("sym" %in% breaks) {
            breaks <- max(abs(x), na.rm = TRUE)
            breaks2 <- breaks/45
            breaks <- seq(-breaks,breaks,breaks2)
        }
        if (length(breaks) == 1) {
            at <- seq(min(x, na.rm = TRUE), max(x, na.rm = TRUE),
            (max(x, na.rm = TRUE)-min(x, na.rm = TRUE))/breaks)
        } else {
            at <- breaks
            x[x < breaks[1]] <- breaks[1]
            x[x > breaks[length(breaks)]] <- breaks[length(breaks)]
        }
        na.idx <- NULL
        if ((Colv | Rowv) & any(is.na(x) == TRUE)) {
            na.idx <- which(is.na(x) == TRUE)
            x[na.idx] <- mean(x, na.rm = TRUE)
        }
        if ((Colv | Rowv) & any(is.na(clusterx) == TRUE)) {
          na.idx.tmp <- which(is.na(clusterx) == TRUE)
          clusterx[na.idx.tmp] <- mean(clusterx, na.rm = TRUE)
        }
        dd.col <- NULL
        if (dendrogram == "row" | dendrogram == "both" & !is.null(colorkey[1])) {
            colorkey = list(space = "left")
        }
        if (Colv) {
            if (is.null(clust[1])) {
                if (is.null(clusterx[1])) {
                    dd.col <- as.dendrogram(hclust(dist(t(x))))
                } else {
                    dd.col <- as.dendrogram(hclust(dist(t(clusterx))))
                }
            } else {
                if (is.null(clusterx[1])) {
                    cor.tmp <- cor(x, method = clust)
                } else {
                    cor.tmp <- cor(clusterx, method = clust)
                }
                cor.tmp[which(is.na(cor.tmp) == TRUE)] <- 0
                dd.col <- as.dendrogram(hclust(as.dist(1-abs(cor.tmp))))
            }
            col.ord <- order.dendrogram(dd.col)
        } else {
            if (dendrogram %in% "both") {
                dendrogram <- "row"
            }
            if (dendrogram %in% "col") {
                dendrogram <- "none"
            }
            col.ord <- 1:ncol(x)
            legend = NULL
        }
        if (Rowv) {
            if (is.null(clust[1])) {
                if (is.null(clusterx[1])) {
                    dd.row <- as.dendrogram(hclust(dist(x)))
                } else {
                    dd.row <- as.dendrogram(hclust(dist(clusterx)))
                }
            } else {
                if (is.null(clusterx[1])) {
                    cor.tmp <- cor(t(x), method = clust)
                } else {
                    cor.tmp <- cor(t(clusterx), method = clust)
                }
                cor.tmp[which(is.na(cor.tmp) == TRUE)] <- 0
                dd.row <- as.dendrogram(hclust(as.dist(1-abs(cor.tmp))))
            }
            row.ord <- rev(order.dendrogram(dd.row))
        } else {
            if (dendrogram %in% "both") {
                dendrogram <- "col"
            }
            if (dendrogram %in% "row") {
                dendrogram <- "none"
            }
            row.ord <- 1:nrow(x)
            legend = NULL
        }
        if (!is.null(na.idx[1])) x[na.idx] <- NA
        add <- list(rect = list(col = "transparent",
                                fill = colSideColors[sort(col.ord)]))
        if (length(col)==1) {
            myTheme <-
                custom.theme(region=RColorBrewer::brewer.pal(n=coln, col))
        } else {
            myTheme <- custom.theme(region=col)
        }
        if (dendrogram != "none") {
            if (dendrogram == "both") {
                if (colSideColorsPos %in% "bottom") {
                    if (!is.null(colSideColors[1])) {
                        size <- 2
                    } else {
                        size <- 10
                    }
                    legend <- list(bottom =
                                       list(fun = dendrogramGrob,
                                            args =
                                                list(x = dd.col, ord = col.ord,
                                                     side = "top",
                                                     size = size,
                                                     add = add,
                                                     type = "rectangle")),
                                   right =
                                       list(fun = dendrogramGrob,
                                            args =
                                                list(x = dd.row,
                                                     side = "right",
                                                     size = 10)))
                }
                if (colSideColorsPos %in% "top") {
                    legend <- list(top =
                                       list(fun = dendrogramGrob,
                                            args =
                                                list(x = dd.col, ord = col.ord,
                                                     side = "top",
                                                     size = 10,
                                                     add = add,
                                                     type = "rectangle")),
                                   right =
                                       list(fun = dendrogramGrob,
                                            args =
                                                list(x = dd.row,
                                                     side = "right",
                                                     size = 10)))
                }
            }
            if (dendrogram == "row") {
                if (!is.null(colSideColors[1])) {
                    if (is.null(dd.col[1])) {
                        col.ord <- 1:length(colSideColors)
                    }
                    if (is.null(clusterx[1])) {
                        dd.col <- as.dendrogram(hclust(dist(t(x))*0))
                    } else {
                        dd.col <- as.dendrogram(hclust(dist(t(clusterx))*0))
                    }
                    if (colSideColorsPos %in% "bottom") {
                        legend <-
                            list(right =
                                     list(fun = dendrogramGrob,
                                          args =
                                              list(x = dd.row,
                                                   side = "right",
                                                   size = 10,
                                                   size.add= 0.5)),
                                 bottom =
                                     list(fun = dendrogramGrob,
                                          args = list(x = dd.col, ord = col.ord,
                                                      side = "top", size = 1,
                                                      size.add= 1,
                                                      add = add,
                                                      type = "rectangle")))
                    }
                    if (colSideColorsPos %in% "top") {
                        legend <-
                            list(right =
                                     list(fun = dendrogramGrob,
                                          args =
                                              list(x = dd.row,
                                                   side = "right",
                                                   size = 10, size.add= 0.5)),
                                 top =
                                     list(fun = dendrogramGrob,
                                          args = list(x = dd.col, ord = col.ord,
                                                      side = "top", size = 1,
                                                      size.add= 1,
                                                      add = add,
                                                      type = "rectangle")))
                    }
                } else {
                    legend <-
                        list(right =
                                 list(fun = dendrogramGrob,
                                      args =
                                          list(x = dd.row,
                                               side = "right",
                                               size = 10, size.add= 0.5)))
                }
            }
            if (dendrogram == "col") {
                if (colSideColorsPos %in% "bottom") {
                    if (!is.null(colSideColors[1])) {
                        size <- 2
                    } else {
                        size <- 10
                    }
                    legend <-
                        list(bottom =
                                 list(fun = dendrogramGrob,
                                      args = list(x = dd.col, ord = col.ord,
                                                  side = "top", size = size,
                                                  size.add= 0.5,
                                                  add = add,
                                                  type = "rectangle")))
                }
                if (colSideColorsPos %in% "top") {
                    legend <-
                        list(top =
                                 list(fun = dendrogramGrob,
                                      args = list(x = dd.col, ord = col.ord,
                                                  side = "top", size = 10,
                                                  size.add= 0.5,
                                                  add = add,
                                                  type = "rectangle")))
                }
            }
        } else {
            if (!is.null(colSideColors[1])) {
                if (is.null(dd.col[1])) {
                    col.ord <- 1:length(colSideColors)
                }
                if (is.null(clusterx[1])) {
                    dtx <- dist(t(x))
                    dtx[is.na(dtx)] <- 0
                    dd.col <- as.dendrogram(hclust(dtx*0))
                } else {
                    dtcx <- dist(t(clusterx))
                    dtcx[is.na(dtcx)] <- 0
                    dd.col <- as.dendrogram(hclust(dtcx*0))
                }
                if (colSideColorsPos %in% "bottom") {
                    legend <-
                        list(bottom =
                                 list(fun = dendrogramGrob,
                                      args = list(x = dd.col, ord = col.ord,
                                                  side = "top", size = 1,
                                                  size.add= 1,
                                                  add = add,
                                                  type = "rectangle"))
                             )
                }
                if (colSideColorsPos %in% "top") {
                    legend <-
                        list(top =
                                 list(fun = dendrogramGrob,
                                      args = list(x = dd.col, ord = col.ord,
                                                  side = "top", size = 1,
                                                  size.add= 1,
                                                  add = add,
                                                  type = "rectangle"))
                             )
                }
            } else {
                legend <- NULL
            }
        }
        d <- t(x[row.ord, col.ord,drop=FALSE])
        d <- d[, ncol(d):1, drop=FALSE]
        if (contour) {
            region <- TRUE
            col.regions <- terrain.colors(100)
        }
        ##  print(p, position=c(0,ypct-0.05,1,1), more=TRUE)
        ##  print(p2, position=c(0,0,1,ypct+0.05))
        if (is.null(rownames(x)[1])) {
            ytck <- list(cex = 0, rot = 0, at = NULL)
        } else {
            ytck <- list(cex = cexRow, rot = yrot)
        }
        if (is.null(colnames(x)[1])) {
            xtck <- list(cex = 0, rot = 0, at = NULL)
        } else {
            xtck <- list(cex = cexCol, rot = xrot)
        }
        legend <- c(legend,legend0)
        levelplot(d,
                  main = list(label = main, cex = cexMain),
                  sub = list(label = sub, cex = cexSub),
                  aspect = aspect,
                  xlab=xlab,
                  ylab=ylab,
                  scales = list(x = xtck, y = ytck, tck = c(1,0)),
                  par.settings=myTheme,
                  border=bordercol,
                  border.lwd=borderwidth,
                  shrink=shrink,
                  legend = legend,
                  at = at,
                  colorkey = colorkey,
                  contour = contour,
                  lattice.options=list(axis.padding=list(factor=axis.padding)),
                  panel = if (useRaster) {
                              function(...) {
                                  panel.fill(col = colNA)
                                  panel.levelplot.raster(...)
                              }
                          } else {
                              function(...) {
                                  panel.fill(col = colNA)
                                  panel.levelplot(...)
                              }
                          }
                  )
    }

#' Analyse large double knock-out screen.
#'
#' This function is used to analyse knock-out screens with multiple
#' double and single knock-outs combined in one data set.
#' @param data data matrix containing multiple single and double knock-downs
#' in columns and effect reporters in the rows
#' @param ... additional parameters, e.g. for the main epiNEM function
#' @export
#' @author Martin Pirkl
#' @return list object with vectors of double knock-downs, single knock-downs
#' and two matrices with doubles in the columns and singles in the rows. The
#' first matrix denotes the respective logical gate for the triple and the
#' second matrix the log-likelihood
#' @examples
#' data <- matrix(sample(c(0,1), 100*9, replace = TRUE), 100, 9)
#' colnames(data) <- c("A.B", "A.C", "B.C", "A", "B", "C", "D", "E", "G")
#' rownames(data) <- paste("E", 1:100, sep = "_")
#' res <- epiScreen(data)
epiScreen <- function(data, ...) {

    dataBin <- data

    doubles <- colnames(dataBin)[grep("\\.", colnames(dataBin))]

    if (length(grep("vs", doubles)) > 0) {
        doubles <- sort(doubles[-grep("vs", doubles)])
    } else { doubles <- sort(doubles) }

    doubles.genes <- unique(unlist(strsplit(doubles, "\\.")))

    if (length(grep("\\.", colnames(dataBin))) > 0) {
        singles <- colnames(dataBin)[-grep("\\.", colnames(dataBin))]
    } else { singles <- sort(singles) }

    singles <- unique(sort(singles))

    llmat <- logicmat <- matrix(0, length(singles), length(doubles))

    rownames(llmat) <- rownames(logicmat) <- singles

    colnames(llmat) <- colnames(logicmat) <- doubles

    globalgenes <- which(apply(dataBin, 1, max) == 1)

    targets <- list()

    for (i in doubles) {
        targets[[i]] <- list()
        print(i)
        doubles.singles <- unlist(strsplit(i, "\\."))
        egenes <-
            which(apply(dataBin[, which(colnames(dataBin) %in%
                                        c(i, doubles.singles))], 1, max) == 1)
        for (j in singles) {
            print(j)
            if (j %in% doubles.singles) { next() }

            dataTmp <- dataBin[, grep(paste(
                paste("^", c(i, j, doubles.singles), "$", sep = ""),
                collapse = "|"),
                colnames(dataBin))]

            dataTmp <- dataTmp[egenes, ]

            i1 <- which(singles %in% j)
            i2 <- which(doubles %in% i)

            if (!(is.null(dim(dataTmp)[1]))) {

                if (any(dataTmp[, j] != 0)) {

                    epires <- epiNEM(dataTmp, method = "exhaustive")

                    targets[[i]][[j]] <-
                        rownames(dataTmp)[which(apply(epires$PostScore, 1,
                                                      which.max) ==
                                                which(colnames(epires$PostScore)
                                                      %in% j))]

                    tmp <- epires$logics
                    if ("OR" %in% tmp) {
                        if (sum(epires$origModel[, j]) != 2) {
                            tmp <- "NOEPI"
                        } else {
                            if (all(tmp %in% "OR")) {
                                tmp <- "OR"
                            } else {
                                tmp <- tmp[which(!(tmp %in% "OR"))]
                            }
                        }
                    }

                    logicmat[i1, i2] <- tmp
                    llmat[i1, i2] <- epires$score

                } else {

                    logicmat[i1, i2] <- "UNCON"
                    llmat[i1, i2] <- -Inf

                }

            } else {

                logicmat[i1, i2] <- "UNCON"
                llmat[i1, i2] <- -Inf

            }

        }

    }

    results <- list(doubles = doubles, singles = singles,
                    logic = logicmat, ll = llmat, targets = targets)

    class(results) <- "epiScreen"

    return(results)

}

#' Gate visualisation.
#'
#' Plots logical gate data annotation. The 8 heatmaps visualize what perfect
#' data would look like in respective to each logical gate. Perfect data is
#' equivalent to Boolean truth tables.
#' @references \url{https://en.wikipedia.org/wiki/Boolean_algebra}
#' @export
#' @author Martin Pirkl
#' @return plot of heatmaps showing the silencing scheme (=expected data,
#' truth tables)
#' @examples
#' epiAnno()
epiAnno <- function() {
    oldw <- getOption("warn")
    options(warn=-1)
    a1 <- HeatmapOP(matrix(c(1,-1,1,-1,1,1, 1, 1, 1), 3, 3,
                           dimnames = list(c("A", "B", "A.B"), LETTERS[1:3])),
                    Colv = FALSE, Rowv = FALSE,
                    main = "OR", col = "Greys", sub = "", colorkey = NULL)
    a2 <- HeatmapOP(matrix(c(1,-1,1,-1,1,1, -1, -1, 1), 3, 3,
                           dimnames = list(c("A", "B", "A.B"), LETTERS[1:3])),
                    Colv = FALSE, Rowv = FALSE,
                    main = "AND", col = "Greys", sub = "", colorkey = NULL)
    a3 <- HeatmapOP(matrix(c(1,-1,1,-1,1,1, 1, -1, -1), 3, 3,
                           dimnames = list(c("A", "B", "A.B"), LETTERS[1:3])),
                    Colv = FALSE, Rowv = FALSE,
                    main = "B masks effect of A", col = "Greys", sub = "",
                    colorkey = NULL)
    a4 <- HeatmapOP(matrix(c(1,-1,1,-1,1,1, -1, 1, -1), 3, 3,
                           dimnames = list(c("A", "B", "A.B"), LETTERS[1:3])),
                    Colv = FALSE, Rowv = FALSE,
                    main = "A masks effect of B", col = "Greys", sub = "",
                    colorkey = NULL)
    a5 <- HeatmapOP(matrix(c(1,-1,1,-1,1,1, 1, 1, -1), 3, 3,
                           dimnames = list(c("A", "B", "A.B"), LETTERS[1:3])),
                    Colv = FALSE, Rowv = FALSE,
                    main = "XOR", col = "Greys", sub = "", colorkey = NULL)
    a6 <- HeatmapOP(matrix(c(1,-1,1,-1,1,1, -1, 1, 1), 3, 3,
                           dimnames = list(c("A", "B", "A.B"), LETTERS[1:3])),
                    Colv = FALSE, Rowv = FALSE,
                    main = "No epistasis", col = "Greys", sub = "",
                    colorkey = NULL)
    a7 <- HeatmapOP(matrix(c(1,-1,1,-1,1,1, 1, -1, 1), 3, 3,
                           dimnames = list(c("A", "B", "A.B"), LETTERS[1:3])),
                    Colv = FALSE, Rowv = FALSE,
                    main = "No epistasis", col = "Greys", sub = "",
                    colorkey = NULL)
    a8 <- HeatmapOP(matrix(c(1,-1,1,-1,1,1, -1, -1, -1), 3, 3,
                           dimnames = list(c("A", "B", "A.B"), LETTERS[1:3])),
                    Colv = FALSE, Rowv = FALSE,
                    main = "No epistasis (unconnected)", col = "Greys",
                    sub = "", colorkey = NULL)
    print(a5, position = c(0,0, .25, .5), more = TRUE)
    print(a6, position = c(.25,0, .5, .5), more = TRUE)
    print(a7, position = c(.5,0, .75, .5), more = TRUE)
    print(a8, position = c(.75,0, 1, .5), more = TRUE)
    print(a1, position = c(0,.5, .25, 1), more = TRUE)
    print(a2, position = c(.25,.5, .5, 1), more = TRUE)
    print(a3, position = c(.5,.5, .75, 1), more = TRUE)
    print(a4, position = c(.75,.5, 1, 1))
    options(warn=oldw)
}

###--- END OF HELPER FUNCTIONS ---###


col = "RdYlGn"; colNA = "grey"; coln = 11; bordercol = "grey";
             borderwidth = 0.1; breaks = "sym";
             main = "";
             sub = "";
             dendrogram = "none"; colorkey = list(space = "right"); Colv = TRUE;
             Rowv = TRUE; xrot = 90; yrot = 0; shrink = c(1,1); cexCol = 1;
             cexRow = 1; cexMain = 1; cexSub = 1;
             colSideColors = NULL; aspect = "fill";
             contour = FALSE; useRaster = FALSE; xlab = NULL; ylab = NULL;
             colSideColorsPos = "top"; clust = NULL; clusterx = NULL;

data <- matrix(sample(c(0,1), 100*4, replace = TRUE), 100, 4)
colnames(data) <- c("A", "A.B", "B", "C")
rownames(data) <- paste("E", 1:100, sep = "_")
res1 <- epiNEM(data, method = "exhaustive")
res2 <- epiNEM(data, method = "greedy", nIterations = 1,
               init = res1$origModel)
checkEquals(res1, res2, checkNames = TRUE)

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/finalFunctions.R
\name{CreateExtendedAdjacency}
\alias{CreateExtendedAdjacency}
\title{Create an extended adjacency matrix}
\usage{
CreateExtendedAdjacency(network, mutants, experiments)
}
\arguments{
\item{network}{network created by BoolNet from file}

\item{mutants}{vector of single knockouts}

\item{experiments}{vector of all knockouts}
}
\value{
extended adjacency matrix
}
\description{
extend adjacency matrices taking cycles and logics into account.
For every given start state, the final state
is computed yu using BoolNet.
}
\examples{
library(BoolNet)
data(cellcycle)
extModel <- CreateExtendedAdjacency(cellcycle,
c(cellcycle$genes, "CycD.Rb"), cellcycle$genes)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_results.R
\name{perm.rank.test}
\alias{perm.rank.test}
\title{AUC permutation test}
\usage{
perm.rank.test(
  x,
  y = NULL,
  alternative = c("two.sided", "less", "greater"),
  iter = 1000
)
}
\arguments{
\item{x}{numeric vector of ranks}

\item{y}{numeric vector of the superset of x}

\item{alternative}{character for test type: 'less','greater','two.sided'}

\item{iter}{integer number of iterations}
}
\value{
p-value
}
\description{
computes the area under the rank enrichment score curve and does
a permutation test to compute the p-value
}
\examples{
x <- 1:10
y <- 1:100
perm.rank.test(x,y,alternative='less')
perm.rank.test(x,y,alternative='greater')
}
\author{
Martin Pirkl
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\name{SimEpiNEM}
\alias{SimEpiNEM}
\title{Compare algorithms.}
\usage{
SimEpiNEM(
  runs = 10,
  do = c("n", "e"),
  random = list(FPrate = 0.1, FNrate = c(0.1, 0.5), single = 3, double = 1, reporters =
    10, replicates = 2),
  maxTime = FALSE,
  forcelogic = TRUE,
  epinemsearch = "greedy",
  bnemsearch = "genetic",
  ...
)
}
\arguments{
\item{runs}{number simulation runs}

\item{do}{string vector of algorithms to
compare: e (epiNEM), n (Nested Effects Models),
b (B-NEM), p (PC algorithm), a (Aracne), e.g. c("e", "n", "p")}

\item{random}{list of false poitive rate FPrate, false negative rates
FNrate, number of single knock-downs single, number of double
knock-downs double, number of effect reporters reporters and number
of replicates replicates}

\item{maxTime}{TRUE if the algorithms are bound to a maximum running
time in respect to epiNEM}

\item{forcelogic}{if TRUE the randomly sampled ground truth network
includes a complex logic with probability 1}

\item{epinemsearch}{greedy or exhaustive search for epiNEM}

\item{bnemsearch}{genetic or greedy search for B-NEM}

\item{...}{additional parameters}
}
\value{
returns list of specificity and sensitivity of inferred edges
(spec, sens) and inferred expected data (spec2, sens2) and accuracy
of logics (logics) and running time (time)
}
\description{
Compares different network reconstruction algorithm on
simulated data.
}
\examples{
res <- SimEpiNEM(runs = 1)
}
\author{
Martin Pirkl
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\name{GenerateData}
\alias{GenerateData}
\title{Generate data from extended model.}
\usage{
GenerateData(model, extTopology, FPrate, FNrate, replicates)
}
\arguments{
\item{model}{model of a topology
from CreateTopology}

\item{extTopology}{extended topology}

\item{FPrate}{false positive rate}

\item{FNrate}{false negative rate}

\item{replicates}{number of replicates}
}
\value{
data matrix with effect reporters as rows and knock-downs
(including double kock-downs) as columns.
}
\description{
Given a model created from
CreateTopology and ExtendTopology, this function creeates acorresponding
artificial data matrix, which is used as a ground truth for simulation
studies.
}
\examples{
topology <-
CreateTopology(3, 1, force = TRUE)
topology <-
unlist(unique(topology), recursive = FALSE)
extTopology <-
ExtendTopology(topology$model, 100)
sortedData <-
GenerateData(topology$model, extTopology, 0.05, 0.13, 3)
}
\seealso{
CreateTopology
}
\author{
Madeline Diekmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\docType{data}
\name{wageningen_string}
\alias{wageningen_string}
\title{sig. of string interaction scores
for van Wageningen et al., 2010 data}
\description{
The data consists of a list including a vectors of pairs (for interactions)
and a corresponding list of interaction scores derived form the
string database.
For details see the vignette.
}
\examples{
data(wageningen_string)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\name{HeatmapOP}
\alias{HeatmapOP}
\title{Heatmap.}
\usage{
HeatmapOP(
  x,
  col = "RdYlGn",
  colNA = "grey",
  coln = 11,
  bordercol = "grey",
  borderwidth = 0.1,
  breaks = "sym",
  main = "",
  sub = "",
  dendrogram = "none",
  colorkey = "right",
  Colv = TRUE,
  Rowv = TRUE,
  xrot = 90,
  yrot = 0,
  shrink = c(1, 1),
  cexCol = 1,
  cexRow = 1,
  cexMain = 1,
  cexSub = 1,
  colSideColors = NULL,
  aspect = "fill",
  contour = FALSE,
  useRaster = FALSE,
  xlab = NULL,
  ylab = NULL,
  colSideColorsPos = "top",
  clust = NULL,
  clusterx = NULL,
  axis.padding = 0.5,
  legend = NULL,
  ...
)
}
\arguments{
\item{x}{Matrix.}

\item{col}{Color. See brewer.pal.info for all available
color schemes. Alternatively, any number of colors, which
 are then used to create a color gradient. E.g., c('blue','red')
  produces a color scheme with a gradient from blue to red.}

\item{colNA}{color for NAs; defaul is grey}

\item{coln}{Number of colors.}

\item{bordercol}{Border color.}

\item{borderwidth}{Border width.}

\item{breaks}{Defines the breaks in the color range. "sym"
makes the breaks symmetric around 0.}

\item{main}{Main title.}

\item{sub}{Subtitle.}

\item{dendrogram}{Draw dendrogram with "both", "col" or
"row", or do not draw with "none".}

\item{colorkey}{Draw colorkey "left", "right" (default), "top", "bottom"
or NULL for no colorkey. See ?lattice::levelplot for more complex colorkey
options.}

\item{Colv}{Cluster columns (TRUE) or not (FALSE).}

\item{Rowv}{Cluster rows (TRUE) or not (FALSE).}

\item{xrot}{Rotate the column names by degree.}

\item{yrot}{Rotate the row names by degree.}

\item{shrink}{c(x,y) defines a range of size for the data
boxes from low to high.}

\item{cexCol}{Font size of column names.}

\item{cexRow}{Font size of row names.}

\item{cexMain}{Font size of main title.}

\item{cexSub}{Font size of subtitle.}

\item{colSideColors}{Defines a numeric vector to annotate
columns with different colors.}

\item{aspect}{"iso" for quadratic boxes or "fill" for
streched boxes.}

\item{contour}{TRUE adds a contour plot.}

\item{useRaster}{TRUE to add raster visuals}

\item{xlab}{Label for the x-axis.}

\item{ylab}{Label for the y-axis.}

\item{colSideColorsPos}{Place colSideColors at the "top" or "bottom".}

\item{clust}{p, s, or k for correlation clustering}

\item{clusterx}{Optional data matrix y with the same dimensions
as x. x's columns or rows are sorted by the cluster information of y.
Col- and rownames of y must be in the same order as in x.}

\item{axis.padding}{padding around the heatmap (0.5 is no padding,
default)}

\item{legend}{list obejct. For parameters see base function ?legend for details. x and y parameters are relative to the inside of the heatmap and are between 0 and 1. E.g., to place the legend outside of the heatmap x and y need to be either less than 0 or greater than 1.}

\item{\dots}{Optional arguments.}
}
\value{
lattice object/matrix
}
\description{
Heatmap function based on the lattice package
more information: ?xyplot
}
\examples{
x <- matrix(rnorm(50), 10, 5)
HeatmapOP(x, dendrogram = "both", aspect = "iso", xrot = 45)
}
\author{
Martin Pirkl & Oscar Perpinan
at http://oscarperpinan.github.io/rastervis/
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\name{epiAnno}
\alias{epiAnno}
\title{Gate visualisation.}
\usage{
epiAnno()
}
\value{
plot of heatmaps showing the silencing scheme (=expected data,
truth tables)
}
\description{
Plots logical gate data annotation. The 8 heatmaps visualize what perfect
data would look like in respective to each logical gate. Perfect data is
equivalent to Boolean truth tables.
}
\examples{
epiAnno()
}
\references{
\url{https://en.wikipedia.org/wiki/Boolean_algebra}
}
\author{
Martin Pirkl
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\name{epiNEM}
\alias{epiNEM}
\title{Epistatic NEMs - main function.}
\usage{
epiNEM(
  filename = "random",
  method = "greedy",
  nIterations = 10,
  nModels = 0,
  random = list(single = 4, double = 1, reporters = 100, FPrate = 0.1, FNrate = 0.1,
    replicates = 1),
  ltype = "marginal",
  para = c(0.13, 0.05),
  init = NULL
)
}
\arguments{
\item{filename}{A binary, tab-delimited matrix.
Columns: single and double knockdowns.
Rows: genes showing effect or not?
Default: random; artificial data is
generated to 'random' specifications}

\item{method}{greedy or exhaustive search.
Default: greedy}

\item{nIterations}{number of iterations.
Default: 10}

\item{nModels}{number of Models. Default: 0}

\item{random}{list specifying how the
data should be generated:
no. of single mutants, no. of
double mutants, no. of reporterGenes,
FP-rate, FN-rate, no. of replicates}

\item{ltype}{likelihood either
"marginal" or "maximum"}

\item{para}{false positive and
false negative rates}

\item{init}{adjacency matrix to initialise the greedy search}
}
\value{
List object with an adjacency matrix denoting the network,
the model of the silencing scheme (rows are knock-downs, columns
are signalling genes), a string with the inferred logial gates,
a column indices denoting position of logical gates, the log transformed
likelihood and the effect reporter distribution (rows are the signalling
genes including the null node).
}
\description{
This function contains the inference
algorithm to learn logical networks from knock-down data including
double knock-downs.
}
\examples{
data <- matrix(sample(c(0,1), 100*4, replace = TRUE), 100, 4)
colnames(data) <- c("A", "A.B", "B", "C")
rownames(data) <- paste("E", 1:100, sep = "_")
res <- epiNEM(data, method = "exhaustive")
plot(res)
}
\seealso{
nem
}
\author{
Madeline Diekmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\docType{data}
\name{wagscreen}
\alias{wagscreen}
\title{Example data: epiNEM results for
the Wageningen et al., 2010 knock-out screen
"http://www.holstegelab.nl/publications/GSTF_geneticinteractions/
downloads/del_mutants_limma.txt"}
\description{
The data consists of a list of matrices with the likelihoods (ll)
for each analysed triple of signalling genes and the inferred logic
(logic) for each triple. The signalling genes or modulators C are the
rows and the signalling genes from the double knock-downs are in the columns.
For details see the vignette.
}
\examples{
data(wagscreen)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_results.R
\name{rank.enrichment}
\alias{rank.enrichment}
\title{Rank enrichment}
\usage{
rank.enrichment(
  data,
  list,
  list2 = NULL,
  n = 1000,
  main = NULL,
  col1 = "RdBu",
  col2 = rgb(1, 0, 0, 0.75),
  col3 = rgb(0, 0, 1, 0.75),
  blim = NULL,
  p = NULL,
  lwd = 3,
  test = wilcox.test,
  vis = "matrix",
  verbose = FALSE,
  ...
)
}
\arguments{
\item{data}{m times l matrix with m observed genes and l
variables with numeric values to rank the genes}

\item{list}{list of of vectors of genes}

\item{list2}{optional list with same length as list}

\item{n}{length of the gradient (maximum: m)}

\item{main}{character string for main header; if NULL uses
the column names of data by default}

\item{col1}{color of the gradient}

\item{col2}{color of the first list}

\item{col3}{color of the second list2}

\item{blim}{numeric vector of length two with the lower
and upper bounds for the gradient}

\item{p}{numeric adjustment (length four) of the left side of the
gradient (low means more to the left, high more to the right)
the right side of the enrichment lines and the top positions
of the additional matrices in case of vis='matrices'}

\item{lwd}{line width of the enrichment lines}

\item{test}{test function for the enrichment p-value; must
have input argument and output values same as perm.rank.test;
e.g., wilcox.test or ks.test (here 'less' and 'greater' are switched!)}

\item{vis}{method for visualisation: 'matrix' uses one
matrix heatmap for; 'matrices' uses several matrices
(experimental), 'colside'
uses the colSideColors argument for the ticks of genes in
list/list2 (can use a lot of memory; experimental)}

\item{verbose}{if TRUE gives prints additional output}

\item{...}{additional arguments for epiNEM::HeatmapOP}
}
\value{
transitively closed matrix or graphNEL
}
\description{
Infers a signalling pathway from peerturbation experiments.
}
\examples{
data <- matrix(rnorm(100*2),100,2)
rownames(data) <- 1:100
colnames(data) <- LETTERS[1:2]
list <- list(first = as.character(sample(1:100, 10)), second = as.character(sample(1:100, 20)))
rank.enrichment(data,list)
}
\author{
Martin Pirkl
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/finalFunctions.R
\name{CreateTopology}
\alias{CreateTopology}
\title{Create Topology.}
\usage{
CreateTopology(single, double, force = TRUE)
}
\arguments{
\item{single}{number of single knockouts}

\item{double}{number of double knockouts}

\item{force}{if true the random model will have a sophisticated logical gate}
}
\value{
adjacency matrix
}
\description{
Create topology for a randomly generated pathway topology
}
\examples{
model <- CreateTopology(3, 1)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\docType{data}
\name{sim}
\alias{sim}
\title{Example data: simulation results}
\description{
Contains simulation results. How they were
aquired is explained in the vignette.
The data conists of a list of data matrices holding
sensitivity and specificity
(spec, sens) of network edges for the variious methods compared to
the ground truth, sensitivity and specificity (sens2, spec2)
of the expected data for epiNEM and Boolean NEMs and accuracy
of the inferred logics for both. The different methods are in the
rows and the columns denote the different independent simulation runs.
}
\examples{
data(sim)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/finalFunctions.R
\name{CreateRandomGraph}
\alias{CreateRandomGraph}
\title{Create a random graph}
\usage{
CreateRandomGraph(pathwayGenes, edgeProb = 0.5)
}
\arguments{
\item{pathwayGenes}{vector of genes in the pathway}

\item{edgeProb}{probability of random edge}
}
\value{
adjacency matrix
}
\description{
Returns a model graph with randomly sampled edges.
Every possible edge has a probability to exist in the graph.
}
\examples{
graph <- CreateRandomGraph(c("Ikk1", "Ikk2", "RelA"))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\name{epiScreen}
\alias{epiScreen}
\title{Analyse large double knock-out screen.}
\usage{
epiScreen(data, ...)
}
\arguments{
\item{data}{data matrix containing multiple single and double knock-downs
in columns and effect reporters in the rows}

\item{...}{additional parameters, e.g. for the main epiNEM function}
}
\value{
list object with vectors of double knock-downs, single knock-downs
and two matrices with doubles in the columns and singles in the rows. The
first matrix denotes the respective logical gate for the triple and the
second matrix the log-likelihood
}
\description{
This function is used to analyse knock-out screens with multiple
double and single knock-outs combined in one data set.
}
\examples{
data <- matrix(sample(c(0,1), 100*9, replace = TRUE), 100, 9)
colnames(data) <- c("A.B", "A.C", "B.C", "A", "B", "C", "D", "E", "G")
rownames(data) <- paste("E", 1:100, sep = "_")
res <- epiScreen(data)
}
\author{
Martin Pirkl
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\docType{data}
\name{sameith_string}
\alias{sameith_string}
\title{sig. of string interaction scores
for Sameith et al., 2015 data}
\description{
The data consists of a list including a vectors of pairs (for interactions)
and a corresponding list of interaction scores derived form the
string database.
For details see the vignette.
}
\examples{
data(sameith_string)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\docType{data}
\name{sameith_GO}
\alias{sameith_GO}
\title{graph-based GO similarity scores, string GO annotations
for Sameith et al., 2015 data}
\description{
The data consists of lists including epiNEM identified and
general similarity scores and GO annotations for each triple.
For details see the vignette.
}
\examples{
data(sameith_GO)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/finalFunctions.R
\name{Mll}
\alias{Mll}
\title{Evaluation of graphs}
\usage{
Mll(Phi, D1, D0, ltype = "marginal", para = c(0.13, 0.05))
}
\arguments{
\item{Phi}{model to be evaluated}

\item{D1}{observed data matrix}

\item{D0}{complementary D1}

\item{ltype}{likelihood type either "marginal" or "maximum"}

\item{para}{false positive and false negative rates}
}
\value{
list with likelihood poster probability, egene positions
}
\description{
Computes marginal log-likelihood
for model Phi given observed data matrix D1
}
\examples{
Phi <- matrix(sample(c(0,1), 9, replace = TRUE), 3, 3)
data <- matrix(sample(c(0,1), 3*10, replace = TRUE), 10, 3)
rownames(Phi) <- colnames(Phi) <- colnames(data) <- c("Ikk1", "Ikk2", "RelA")
score <- Mll(Phi, D1 <- data, D0 <- 1 - data)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_results.R
\name{plot.epiNEM}
\alias{plot.epiNEM}
\title{Plot pathway.}
\usage{
\method{plot}{epiNEM}(x, ...)
}
\arguments{
\item{x}{object of class epiNEM}

\item{...}{other arguments}
}
\value{
plot of the logical network
}
\description{
Plots the winning pathway structure
}
\examples{
data <- matrix(sample(c(0,1), 100*4, replace = TRUE), 100, 4)
colnames(data) <- c("A", "A.B", "B", "C")
rownames(data) <- paste("E", 1:100, sep = "_")
res <- epiNEM(data, method = "exhaustive")
plot(res)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\name{ExtendTopology}
\alias{ExtendTopology}
\title{Extending topology of normal "nem"}
\usage{
ExtendTopology(topology, nReporters)
}
\arguments{
\item{topology}{model of a topology from CreateTopology}

\item{nReporters}{number of effects reporters}
}
\value{
extended topology in which reporters
are linked to pathway genes
}
\description{
Extending topology of normal "nem"
}
\examples{
topology <- CreateTopology(3, 1, force = TRUE)
topology <- unlist(unique(topology), recursive = FALSE)
extTopology <- ExtendTopology(topology$model, 100)
}
\seealso{
CreateTopology
}
\author{
Madeline Diekmann
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_results.R
\name{plot.epiScreen}
\alias{plot.epiScreen}
\title{Plot screen.}
\usage{
\method{plot}{epiScreen}(
  x,
  global = TRUE,
  ind = NULL,
  colorkey = TRUE,
  cexGene = 1,
  off = 0.05,
  cexLegend = 1,
  ...
)
}
\arguments{
\item{x}{object of class epiScreen}

\item{global}{plot global distribution or for each pair (FALSE)}

\item{ind}{index of pairs to plot}

\item{colorkey}{if TRUE prints colorkey}

\item{cexGene}{size of modulator annotation}

\item{off}{relative distance from the gene names to the respective
likelihoods}

\item{cexLegend}{font size of the legend}

\item{...}{other arguments}
}
\value{
plot(s) of an epiNEM screen analysis
}
\description{
Plots the sresults of a systematic knock-out screen
}
\examples{
data <- matrix(sample(c(0,1), 100*9, replace = TRUE), 100, 9)
colnames(data) <- c("A.B", "A.C", "B.C", "A", "B", "C", "D", "E", "G")
rownames(data) <- paste("E", 1:100, sep = "_")
res <- epiScreen(data)
plot(res)
plot(res, global = FALSE, ind = 1:3)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_results.R
\name{plot.epiSim}
\alias{plot.epiSim}
\title{Plot simulations.}
\usage{
\method{plot}{epiSim}(x, ...)
}
\arguments{
\item{x}{object of class epiSim}

\item{...}{other arguments}
}
\value{
plot(s) of an epiNEM simulation analysis
}
\description{
Plots the simulation results
}
\examples{
res <- SimEpiNEM(runs = 1)
plot(res)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epiN.R
\docType{data}
\name{samscreen}
\alias{samscreen}
\title{Example data: epiNEM results for
the Sameith et al., 2015 knock-out screen}
\description{
The result of the epiNEM analysis of the data from
"http://www.holstegelab.nl/publications/
sv/signaling_redundancy/downloads/DataS1.txt".
The data consists of a list of matrices with the likelihoods (ll)
for each analysed triple of signalling genes and the inferred logic
(logic) for each triple. The signalling genes or modulators C are the
rows and the signalling genes from the double knock-downs are in the columns.
For details see the vignette.
}
\examples{
data(samscreen)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plot_results.R
\name{AddLogicGates}
\alias{AddLogicGates}
\title{Add logic.}
\usage{
AddLogicGates(child, logic, model)
}
\arguments{
\item{child}{define the child}

\item{logic}{define the logical gate}

\item{model}{normal model}
}
\value{
model list with additional logic gate
}
\description{
extend model with node representing logic gate
}
\examples{
model <- CreateRandomGraph(c("Ikk1", "Ikk2", "RelA"))
model2 <- AddLogicGates("RelA", "OR", model)
}
